#include "Riostream.h"
#include "TROOT.h"
#include "TView.h"
#include "Trapezoid.h"
#include "TVirtualPad.h"
#include "TH1.h"
#include "TH3.h"
#include "TBuffer3D.h"
#include "TBuffer3DTypes.h"
#include "TVirtualViewer3D.h"
#include "TGeometry.h"
#include "TClass.h"
#include "TMath.h"

#include <assert.h>
#include "ropecmds.h"

extern "C" { void strnam_( int * n, char * c );}
extern "C" { void odpblk_( const char * c,  int * n );}
extern "C" { void odpsum_( const char * c,  int * n,   Float_t *x, Float_t *y, Float_t *z);}

extern int gODpbSwitch;
//extern  const int gMaxIntOD;
//extern  const int gMaxRealOD;
extern TString gBlockName;
extern Int_t gBlockNumber;

ClassImp(Trapezoid)
//______________________________________________________________________________
//
// stolen from ROOTs TMarker3DBox

//______________________________________________________________________________
Trapezoid::Trapezoid()
{
   // Marker3DBox  default constructor

   fRefObject = 0;
	for (Int_t i = 0; i < 8; i++ ) {
		fX[i] = 0;
		fY[i] = 0;
		fZ[i] = 0;
	}
	fTransparency = 0;
   SetBit(kTemporary,kFALSE);
}

//______________________________________________________________________________
Trapezoid::Trapezoid( Float_t *x, Float_t *y, Float_t *z,
                      Color_t color, Int_t transp)
              :  TAttLine(color,1,1), TAttFill(color,0), fTransparency(transp)
{
// Marker3DBox normal constructor
//
//        3 ___________________7
//        /___________________/|
//      2| |                6| |
//       | |                 | |
//       | |                 | |
//       | |                 | |
//       | |4________________|_|8
//     1 |/__________________|/
//                           5
	for (Int_t i = 0; i < 8; i++ ) {
		fX[i] = x[i];
		fY[i] = y[i];
		fZ[i] = z[i];
	}
   fRefObject = 0;
   SetBit(kTemporary,kFALSE);
}

//______________________________________________________________________________
//copy constructor
Trapezoid::Trapezoid(const Trapezoid& m3d) :
	TObject(m3d),
	TAttLine(m3d),
	TAttFill(m3d),
	TAtt3D(m3d),
	fTransparency(m3d.fTransparency),
	fRefObject(m3d.fRefObject)
{
	for (Int_t i=0; i< 8; i++) {
		fX[i] = m3d.fX[i];
		fY[i] = m3d.fY[i];
		fZ[i] = m3d.fZ[i];
	}
}

//______________________________________________________________________________
Trapezoid& Trapezoid::operator=(const Trapezoid& m3d)
{
   //assignement operator
   if(this!=&m3d) {
      TObject::operator=(m3d);
      TAttLine::operator=(m3d);
      TAttFill::operator=(m3d);
      TAtt3D::operator=(m3d);
		for (Int_t i=0; i< 8; i++) {
			fX[i] = m3d.fX[i];
			fY[i] = m3d.fY[i];
			fZ[i] = m3d.fZ[i];
		}
      fTransparency=m3d.fTransparency;
      fRefObject=m3d.fRefObject;
   }
   return *this;
}

//______________________________________________________________________________
Trapezoid::~Trapezoid()
{
   // Marker3DBox shape default destructor
}


//______________________________________________________________________________
Int_t Trapezoid::DistancetoPrimitive(Int_t px, Int_t py)
{
   // Compute distance from point px,py to a Marker3DBox
   //
   // Compute the closest distance of approach from point px,py to each corner
   // point of the Marker3DBox.

   const Int_t numPoints = 8;
   Int_t dist = 9999;
   Double_t points[3*numPoints];

   TView *view = gPad->GetView();
   if (!view) return dist;
   const Int_t seg1[12] = {0,1,2,3,4,5,6,7,0,1,2,3};
   const Int_t seg2[12] = {1,2,3,0,5,6,7,4,4,5,6,7};

   SetPoints(points);

   Int_t i, i1, i2, dsegment;
   Double_t x1,y1,x2,y2;
   Double_t xndc[3];
   for (i = 0; i < 12; i++) {
      i1 = 3*seg1[i];
      view->WCtoNDC(&points[i1], xndc);
      x1 = xndc[0];
      y1 = xndc[1];

      i2 = 3*seg2[i];
      view->WCtoNDC(&points[i2], xndc);
      x2 = xndc[0];
      y2 = xndc[1];
      dsegment = DistancetoLine(px,py,x1,y1,x2,y2);
      if (dsegment < dist) dist = dsegment;
   }
   if (dist < 5) {
      gPad->SetCursor(kCross);
      if (fRefObject) {gPad->SetSelected(fRefObject); return 0;}
   }
   return dist;
}


//______________________________________________________________________________
void Trapezoid::ExecuteEvent(Int_t event, Int_t px, Int_t py)
{
   // Execute action corresponding to one event
   //
   // This member function must be implemented to realize the action
   // corresponding to the mouse click on the object in the window

   if (gPad->GetView()) gPad->GetView()->ExecuteRotateView(event, px, py);
}


//______________________________________________________________________________
void Trapezoid::Paint(Option_t * /* option */ )
{
   // Paint marker 3D box.

   static TBuffer3D buffer(TBuffer3DTypes::kGeneric);

   buffer.ClearSectionsValid();

   // Section kCore

   // If we are just a temporary object then no 'real object' to
   // pass to viewer
   if (TestBit(kTemporary)) {
      buffer.fID = 0;
   } else {
      buffer.fID           = this;
   }
   buffer.fColor        = GetLineColor();
   buffer.fTransparency = fTransparency;
   buffer.fLocalFrame   = kFALSE;
   buffer.SetSectionsValid(TBuffer3D::kCore);

   // We fill kCore and kRawSizes on first pass and try with viewer
   TVirtualViewer3D * viewer3D = gPad->GetViewer3D();
   if (!viewer3D) return;
   Int_t reqSections = viewer3D->AddObject(buffer);
   if (reqSections == TBuffer3D::kNone) {
      return;
   }

   if (reqSections & TBuffer3D::kRawSizes) {
      Int_t nbPnts = 8;
      Int_t nbSegs = 12;
      Int_t nbPols = 6;
      if (!buffer.SetRawSizes(nbPnts, 3*nbPnts, nbSegs, 3*nbSegs, nbPols, 6*nbPols)) {
         return;
      }
      buffer.SetSectionsValid(TBuffer3D::kRawSizes);
   }

   if ((reqSections & TBuffer3D::kRaw) && buffer.SectionsValid(TBuffer3D::kRawSizes)) {
      // Points
      SetPoints(buffer.fPnts);

      // Transform points
      if (gGeometry && !buffer.fLocalFrame) {
         Double_t dlocal[3];
         Double_t dmaster[3];
         for (UInt_t j=0; j<buffer.NbPnts(); j++) {
            dlocal[0] = buffer.fPnts[3*j];
            dlocal[1] = buffer.fPnts[3*j+1];
            dlocal[2] = buffer.fPnts[3*j+2];
            gGeometry->Local2Master(&dlocal[0],&dmaster[0]);
            buffer.fPnts[3*j]   = dmaster[0];
            buffer.fPnts[3*j+1] = dmaster[1];
            buffer.fPnts[3*j+2] = dmaster[2];
         }
      }

      // Basic colors: 0, 1, ... 8
      Int_t c = (((GetLineColor()) %8) -1) * 4;
      if (c < 0) c = 0;

      // Segments
      buffer.fSegs[ 0] = c   ; buffer.fSegs[ 1] = 0 ; buffer.fSegs[ 2] = 1;
      buffer.fSegs[ 3] = c+1 ; buffer.fSegs[ 4] = 1 ; buffer.fSegs[ 5] = 2;
      buffer.fSegs[ 6] = c+1 ; buffer.fSegs[ 7] = 2 ; buffer.fSegs[ 8] = 3;
      buffer.fSegs[ 9] = c   ; buffer.fSegs[10] = 3 ; buffer.fSegs[11] = 0;
      buffer.fSegs[12] = c+2 ; buffer.fSegs[13] = 4 ; buffer.fSegs[14] = 5;
      buffer.fSegs[15] = c+2 ; buffer.fSegs[16] = 5 ; buffer.fSegs[17] = 6;
      buffer.fSegs[18] = c+3 ; buffer.fSegs[19] = 6 ; buffer.fSegs[20] = 7;
      buffer.fSegs[21] = c+3 ; buffer.fSegs[22] = 7 ; buffer.fSegs[23] = 4;
      buffer.fSegs[24] = c   ; buffer.fSegs[25] = 0 ; buffer.fSegs[26] = 4;
      buffer.fSegs[27] = c+2 ; buffer.fSegs[28] = 1 ; buffer.fSegs[29] = 5;
      buffer.fSegs[30] = c+1 ; buffer.fSegs[31] = 2 ; buffer.fSegs[32] = 6;
      buffer.fSegs[33] = c+3 ; buffer.fSegs[34] = 3 ; buffer.fSegs[35] = 7;

      // Polygons
      buffer.fPols[ 0] = c   ; buffer.fPols[ 1] = 4 ; buffer.fPols[ 2] = 0;
      buffer.fPols[ 3] = 9   ; buffer.fPols[ 4] = 4 ; buffer.fPols[ 5] = 8;
      buffer.fPols[ 6] = c+1 ; buffer.fPols[ 7] = 4 ; buffer.fPols[ 8] = 1;
      buffer.fPols[ 9] = 10  ; buffer.fPols[10] = 5 ; buffer.fPols[11] = 9;
      buffer.fPols[12] = c   ; buffer.fPols[13] = 4 ; buffer.fPols[14] = 2;
      buffer.fPols[15] = 11  ; buffer.fPols[16] = 6 ; buffer.fPols[17] = 10;
      buffer.fPols[18] = c+1 ; buffer.fPols[19] = 4 ; buffer.fPols[20] = 3;
      buffer.fPols[21] = 8   ; buffer.fPols[22] = 7 ; buffer.fPols[23] = 11;
      buffer.fPols[24] = c+2 ; buffer.fPols[25] = 4 ; buffer.fPols[26] = 0;
      buffer.fPols[27] = 3   ; buffer.fPols[28] = 2 ; buffer.fPols[29] = 1;
      buffer.fPols[30] = c+3 ; buffer.fPols[31] = 4 ; buffer.fPols[32] = 4;
      buffer.fPols[33] = 5   ; buffer.fPols[34] = 6 ; buffer.fPols[35] = 7;

      buffer.SetSectionsValid(TBuffer3D::kRaw);

      TAttLine::Modify();
      TAttFill::Modify();
   }

   viewer3D->AddObject(buffer);
}


//______________________________________________________________________________
// void Trapezoid::SavePrimitive(ostream &out, Option_t * /*= ""*/)
// {
//    // Save primitive as a C++ statement(s) on output stream out
// 
//    out<<"   "<<endl;
//    if (gROOT->ClassSaved(Trapezoid::Class())) {
//       out<<"   ";
//    } else {
//       out<<"   Trapezoid *";
//    }
//    out<<"marker3DBox = new Trapezoid("<<fX<<","
//                                          <<fY<<","
//                                          <<fZ<<","
//                                          <<fDx<<","
//                                          <<fDy<<","
//                                          <<fDz<<","
//                                          <<fTheta<<","
//                                          <<fPhi<<");"<<endl;
// 
//    SaveLineAttributes(out,"marker3DBox",1,1,1);
//    SaveFillAttributes(out,"marker3DBox",1,0);
// 
//    out<<"   marker3DBox->Draw();"<<endl;
// }


//______________________________________________________________________________
void Trapezoid::SetPoints(Double_t *points) const
{
   // Set points.

   if (points) {
      points[ 0] = fX[0]; points[ 1] = fY[0]; points[ 2] = fZ[0];
      points[ 3] = fX[1]; points[ 4] = fY[1]; points[ 5] = fZ[1];
      points[ 6] = fX[2]; points[ 7] = fY[2]; points[ 8] = fZ[2];
      points[ 9] = fX[3]; points[10] = fY[3]; points[11] = fZ[3];
      points[12] = fX[4]; points[13] = fY[4]; points[14] = fZ[4];
      points[15] = fX[5]; points[16] = fY[5]; points[17] = fZ[5];
      points[18] = fX[6]; points[19] = fY[6]; points[20] = fZ[6];
      points[21] = fX[7]; points[22] = fY[7]; points[23] = fZ[7];
//       points[ 0] = -fDx ; points[ 1] = -fDy ; points[ 2] = -fDz;
//       points[ 3] = -fDx ; points[ 4] =  fDy ; points[ 5] = -fDz;
//       points[ 6] =  fDx ; points[ 7] =  fDy ; points[ 8] = -fDz;
//       points[ 9] =  fDx ; points[10] = -fDy ; points[11] = -fDz;
//       points[12] = -fDx ; points[13] = -fDy ; points[14] =  fDz;
//       points[15] = -fDx ; points[16] =  fDy ; points[17] =  fDz;
//       points[18] =  fDx ; points[19] =  fDy ; points[20] =  fDz;
//       points[21] =  fDx ; points[22] = -fDy ; points[23] =  fDz;
   }
}

void Trapezoid::PrintDataDocumented()
{
	std::cout << "fStrNr " << fStrNr << " fElmId " << fElmId 
	<< " Name " << fName << " Title " << fTitle << std::endl;
	TString blockname = fName(4,4);
	gRC->printod_withdoc(blockname, fElmId);
	
}
void Trapezoid::PrintData()
{
	std::cout << "PrintData: fStrNr " << fStrNr << " fElmId " << fElmId 
	<< " Name " << fName << " Title " << fTitle << std::endl;
	TString det = fName(4,4);
	gODpbSwitch = 0;
	odpblk_(det.Data(), &fElmId);
}
;
void Trapezoid::PrintDataTerse()
{
	std::cout << "PrintData terse: fStrNr " << fStrNr << " fElmId " << fElmId 
	<< " Name " << fName << " Title " << fTitle << std::endl;
	TString det = fName(4,4);
	Float_t x=0, y=0, z=0;
	odpsum_(det.Data(), &fElmId, &x, &y, &z);
}

//______________________________________________________________________________
// void Trapezoid::Streamer(TBuffer &R__b)
// {
//    // Stream an object of class Trapezoid.
// 
//    if (R__b.IsReading()) {
//       UInt_t R__s, R__c;
//       Version_t R__v = R__b.ReadVersion(&R__s, &R__c);
//       if (R__v > 1) {
//          R__b.ReadClassBuffer(Trapezoid::Class(), this, R__v, R__s, R__c);
//          return;
//       }
//       //====process old versions before automatic schema evolution
//       TObject::Streamer(R__b);
//       TAttLine::Streamer(R__b);
//       TAttFill::Streamer(R__b);
//       if (R__b.GetVersionOwner() > 22300) {
//          TAtt3D::Streamer(R__b);
//       } else {
//          TAtt3D::Streamer(R__b);
//       }
//       R__b >> fX;
//       R__b >> fY;
//       R__b >> fZ;
//       R__b >> fDx;
//       R__b >> fDy;
//       R__b >> fDz;
//       R__b >> fTheta;
//       R__b >> fPhi;
//       R__b >> fRefObject;
//       R__b.CheckByteCount(R__s, R__c, Trapezoid::IsA());
//       //====end of old versions
// 
//    } else {
//       R__b.WriteClassBuffer(Trapezoid::Class(),this);
//    }
// }
