#ifndef ROOT_Trapezoid
#define ROOT_Trapezoid


////////////////////////////////////////////////////////////////////////////
//                                                                        //
// Trapezoid                                                           	  //
//                                                                        //                                                                        //
////////////////////////////////////////////////////////////////////////////

#include "TObject.h"
#include "TAttLine.h"
#include "TAttFill.h"
#include "TAtt3D.h"

class Trapezoid : public TObject, public TAttLine, public TAttFill,
                     public TAtt3D {
protected:
	Int_t fStrNr;
	Int_t fElmId;
	TString fName;
	TString fTitle;
   Float_t  fX[8];               // X coordinate of corners
   Float_t  fY[8];               // Y coordinate of corners
   Float_t  fZ[8];               // Z coordinate of corners
	Short_t    fTransparency;
   TObject *fRefObject;       // Pointer to an object
   Trapezoid(const Trapezoid&); 
   Trapezoid& operator=(const Trapezoid&);

   enum { kTemporary = BIT(23) }; // Use TObject::fBits to record if we are temporary

public:
   Trapezoid();
   Trapezoid(Float_t *x, Float_t *y, Float_t *z,
                      Color_t color =1, Int_t transp=0);
   virtual        ~Trapezoid();
	void SetStrNr(Int_t seg) { fStrNr = seg; };
	void SetElmId(Int_t id) { fElmId = id; };
	Int_t GetStrNr() { return fStrNr; };
	Int_t GetElmId() { return fElmId; };
	void PrintData();      //*MENU*
	void PrintDataTerse();      //*MENU*
	void PrintDataDocumented();      //*MENU*
	void SetName(const char * name) {fName = name;};  //*MENU*     
	const char * GetName() const {return fName.Data();};   //*MENU*    
	void SetTitle(const char * name) {fTitle = name;};   //*MENU*    
	const char * GetTitle() const {return fTitle.Data();};   //*MENU*    
	virtual Short_t GetTransparency() {return fTransparency;};
	virtual void    SetTransparency(Short_t transp) {fTransparency = transp;};//*MENU*
   virtual Int_t   DistancetoPrimitive(Int_t px, Int_t py);
   virtual void    ExecuteEvent(Int_t event, Int_t px, Int_t py);
   TObject        *GetRefObject() const {return fRefObject;}
   virtual void    Paint(Option_t *option);
//   virtual void    SavePrimitive(ostream &out, Option_t *option = "");
   virtual void    SetPoints(Double_t *buff) const;
   virtual void    SetRefObject(TObject *obj=0) {fRefObject = obj;}

   ClassDef(Trapezoid,0)  //A special 3-D marker designed for event display
};

#endif
