//
// These routines replace the GPHIGS 3D graphics library of GROPE
// by calls to ROOT routines
// 
// Essentially only 3D polylines and markers  are implemented 
// (ppl3 -> TPol3D, ppm3 -> TPolyMarker3D)
// 3D filling and text may be added if really desired
// 
// The 2D calls (polylines, fill area) use HIGZ routines
// which are replaced by ROOT calls in their own higz2root.cxx
//
// A root macro make_skel_gphigs2root.C has been used to generate
// the function bodies automatically from the list of undefined 
// gphigs symbols when linking without the library and the documention
// of GPHIGSs FORTRAN calling sequences.
//
// Disclaimer: All this is tailored to GROPE and may not be regarded
//             as a general replacement of GPHIGS.
//             Of the about 400 GPHIGS routines only 80 are 
//             implemented here of which most are just placeholders

#include "TCanvas.h"
#include "TLatex.h"
#include "TView3D.h"
#include "TString.h"
#include "TMath.h"
#include "Trapezoid.h"
#include "TPol3D.h"
#include "TText3D.h"
#include "TPolyMarker3D.h"
#include "TRegexp.h"
#include "TStyle.h"
#include "TSystem.h"
#include "stdio.h"
//#include "kuip2root.h"
#include <iostream>

class TGLViewer;

extern TCanvas * gCanvas3D;
TLatex * gScaleText = NULL;
Float_t gScale = 100.;

extern int gODpbSwitch;
extern int gColorTracks;
extern float gThickenGeom;
extern float gThickenOD;

extern Int_t kNColDet;
extern Int_t * gColDet;
extern char * gDetName;

extern Int_t gBlockEdges;     // draw edges only
extern Int_t gBlockFill;      // fill only,
extern Int_t gBlockEdgesFill; // both
extern Int_t gTransparency; 

//extern  const int gMaxIntOD;
//extern  const int gMaxRealOD;

extern int gIntOD[500];
extern int gNumberIntOD;
extern float gRealOD[500];
extern int gNumberRealOD;

extern "C" { void strnam_( int * n, char * c );}
extern "C" { void ipl_( int * n, float *x, float *y );}
extern "C" { void ifa_( int * n, float *x, float *y );}
extern "C" { void itx_c_( float *x, float *y, char * c);}
extern "C" { void ischh_( float *x);}
extern "C" { void isplci_( int *col);}
extern "C" { void isfaci_( int *col);}
extern "C" { void istxal_( int *ha, int *va);}
extern "C" { void istxfp_( int *font, int *prec); }
extern "C" { void odpblk_( const char * c,  int * n );}
extern "C" { void view3d_();}

extern "C" {

int gTalign_h = 1;
int gTalign_v = 1;
int gLcolour = 1;
int gPfacolour = 4;
int gPmcolour = 1;
int gPltype = 1;
float gPlwidth = 1;
int gTcolour  = 1;
float gMarkSize =1.;
float gCharH = 0.015;

int gStrnr, gElmid;

Color_t kColUndef=0;
//kColMyon=kCyan, kColProton=kRed, kColKaon=kGreen, kColPi=kBlue, 
//		kColElectron=kMagenta, kColUndef=0;
Color_t PartCol[5] ={6, 7, 4, 3, 2};
//Color_t PartCol[5] ={kMagenta, kCyan, kBlue, kGreen, kRed};
extern Double_t rx0, rx1, ry0, ry1, rz0, rz1;

//____________________________________________________

void ppl3_(int *N, float *PXA, float *PYA, float *PZA);

//void popwk_(int *WKID = 0 , int *CONID = 0 , int *WTYPE = 0);

//____________________________________________________

void pads_(int N, int *NAMSET) 
{
//   ADD NAMES TO SET
}
//____________________________________________________

void pcelst_(int STRID) 
{
//   COPY ALL ELEMENTS FROM STRUCTURE
}
//____________________________________________________

void pclph_() 
{
//   CLOSE PHIGS
}
//____________________________________________________

void pclst_() 
{
//   CLOSE STRUCTURE
}
//____________________________________________________

void pclwk_(int WKID) 
{
//   CLOSE WORKSTATION
}
//____________________________________________________

void pdas_() 
{
//   DELETE ALL STRUCTURES

//	printf("=================  pdas_\n");
	gTalign_h = 1;
	gTalign_v = 1;
	gLcolour = 1;
	gTcolour  = 1;
	
	if ( gCanvas3D ) {
		gCanvas3D->Clear();
		gCanvas3D->cd();
		view3d_();
	} else {
//		popwk_();
	}
}
//____________________________________________________

void pdst_(int *STRID) 
{
	//   DELETE STRUCTURE
//	printf("=================  pdst_\n");
}
//____________________________________________________

void pels_(int STRID, int STRTEP, int SRCDIR, int EISN, int *EIS, int EESN, int *EES, int ERRIND, int STATUS, int FNDEP) 
{
//   ELEMENT SEARCH
}
//____________________________________________________

void pevmm3_(float *VWWNLM, float *PJVPLM, int PJTYPE, float PJRX, float PJRY, float PJRZ, float VPLD, float BPLD, float FPLD, int ERRIND, float *VWMPMT) 
{
//   EVALUATE VIEW MAPPING MATRIX 3
}
//____________________________________________________

void pevom3_(float VWRX, float VWRY, float VWRZ, float VPNX, float VPNY, float VPNZ, float VUPX, float VUPY, float VUPZ, int ERRIND, float *VWORMT) 
{
//   EVALUATE VIEW ORIENTATION MATRIX 3
}
//____________________________________________________

void pfa3_(int *N, float *PXA, float *PYA, float *PZA) 
{
//   FILL AREA 3
//	gCanvas3D->cd();
//	gPlwidth *=3;
	if ( *N == 41 )
	printf("=================  pfa3 %d %f %f %f\n", *N, PXA[0], PYA[0], PZA[0] );
//	gLcolour += 1;

/*	ppl3_(N,PXA,PYA,PZA);
	if ( gLcolour >2 && *N < 10) {
		printf("=================  pfa3 %d color %d \n", *N, gLcolour );
		for (int i = 0; i < *N; i++) {
			printf ("x,y,z %f %f %f \n", PXA[i], PYA[i], PZA[i] );
		}
	}*/
}
//____________________________________________________

void pfa_(int *N, float *PXA, float *PYA) 
{
	gCanvas3D->cd();
	Float_t x[1000], y[1000];
	Int_t np = *N;
	if (np > 1000)
		np = 1000;
	for (Int_t i=0; i < np; i++) {
		Float_t xt = 0.5 * PXA[i];
		Float_t yt = 0.5 * PYA[i];
		x[i] = 20. * (xt - rx0) / (rx1 - rx0);
		y[i] = 20. * (yt - ry0) / (ry1 - ry0);
	}
	ifa_(&np, x, y);
}
//____________________________________________________

void pflush_(int WKID, int ICL, int IDNR) 
{
//   FLUSH DEVICE EVENTS
}
//____________________________________________________

void pgtch_(int STAT, int CHNR) 
{
//   GET CHOICE
}
//____________________________________________________

void pgtvl_(float VAL) 
{
//   GET VALUATOR
}
//____________________________________________________

void pinch3_(int WKID, int CHDNR, int ISTAT, int ICHNR, int PET, float *EVOL, int LDR, char *DATREC) 
{
//   INITIALIZE CHOICE 3
}
//____________________________________________________

void pinlc3_(int WKID, int LCDNR, int IVIEWI, float IPX, float IPY, float IPZ, int PET, float *EVOL, int LDR, char *DATREC) 
{
//   INITIALIZE LOCATOR 3
}
//____________________________________________________

void pinst3_(int WKID, int STDNR, int LSTR, char *ISTR, int PET, float *EVOL, int LDR, char *DATREC) 
{
//   INITIALIZE STRING 3
}
//____________________________________________________

void pinst_(int WKID, int STDNR, int LSTR, char *ISTR, int PET, float XMIN, float XMAX, float YMIN, float YMAX, int LDR, char *DATREC) 
{
//   INITIALIZE STRING
}
//____________________________________________________

void pinvl3_(int WKID, int VLDNR, float IVAL, int PET, float *EVOL, int LDR, char *DATREC) 
{
//   INITIALIZE VALUATOR 3
}
//____________________________________________________

void plb_(int LABEL) 
{
//   LABEL
}
//____________________________________________________

void pmsg_(int WKID, char *MESS) 
{
//   MESSAGE
}
//____________________________________________________

void popph_(int * ERRFIL, int * BUFA) 
{
//   OPEN PHIGS
}
//____________________________________________________

void popst_(int * STRID) 
{
//   OPEN STRUCTURE
//	printf("=================  popst_ %d \n", *STRID);
	gStrnr = *STRID;
}
//____________________________________________________

void popwk_(int *WKID, int *CONID, int *WTYPE) 
{
//   OPEN WORKSTATION
//	printf("=================  popwk_\n");
}
//____________________________________________________

void ppl3_(int *N, float *PXA, float *PYA, float *PZA) 
{
//   POLYLINE 3

	// TPol3D is a TPolyline3D with additional information on 
	// structure number given by the GPHIGS call  popst_ (OPEN STRUCTURE)
	// and Element ID given by pspkid_ (SET PICK IDENTIFIER)
	// These are used to identify tracks / clusters e.g. in calls to  odpblk_
	// OD print data block
	//
//	if (*N == 2) 
//	printf("ppl3 : %d %d %f %f\n",*N, gLcolour, PXA[0], PXA[1] );
	gCanvas3D->cd();
	char strn[100];
	strnam_(&gStrnr, strn);
	TString name (strn);
	name.Resize(8);
	TString det = name(4,4);
	TRegexp space(" ");
	while (name.Index(" ") >= 0) {
		name(space) = "_";
	}
//	printf(" ppl3_ npoints, det name col %d %s %d %f\n", *N,  det.Data(), gLcolour,gPlwidth);
	int col = gLcolour;
//	int pidcolor = gLcolour;
	Bool_t is_cal =  (det == "ECAL" || det == "HCAL" || det == "TOF ");

//	if (det == "ECAL" ) 
//		col = 4;
	Double_t pE = -1;
	if ( det == "ECAL" || det == "HCAL" || det == "CTRK") {
		gODpbSwitch = 1;
		odpblk_(det.Data(), &gElmid);
		if (gNumberRealOD < 1) {
			printf("Not enough words for %s %d\n", det.Data(), gNumberRealOD);
			return;
		}
		if ( det = "CTRK") {
			pE = 0;
			for (Int_t i=0; i < 3; i++) {
				pE += gRealOD[i]*gRealOD[i];
			}
			pE = TMath::Sqrt(pE);
//			printf(" ppl3_ det= %s %f\n",  det.Data(), pE);
		} else {
			pE = gRealOD[0];
		}	
		gODpbSwitch = 0;
	}
	
	TString title(name);
	title+="_";
	title+=gElmid;
	if ( pE > 0 ) {
		title+= "_";
		title+= Form("%4.4g",pE);
		title+= " GeV";
	}
//	printf(" ppl3_ det= %d %d % d %s\n", gBlockEdgesFill, gBlockFill,  gBlockEdges, det.Data());

	
	if ( is_cal && *N == 16 && (gBlockFill || gBlockEdgesFill) ) {
//		printf("=================  ppl3 %d color %d \n", *N, gLcolour );
//		for (int i = 0; i < *N; i++) {
//			printf ("x,y,z %f %f %f \n", PXA[i], PYA[i], PZA[i] );
//		}
		float x[8], y[8], z[8];
		x[0] = PXA[9]; y[0] = PYA[9]; z[0] = PZA[9];
		x[1] = PXA[6]; y[1] = PYA[6]; z[1] = PZA[6];
		x[2] = PXA[1]; y[2] = PYA[1]; z[2] = PZA[1];
		x[3] = PXA[2]; y[3] = PYA[2]; z[3] = PZA[2];
		x[4] = PXA[12]; y[4] = PYA[12]; z[4] = PZA[12];
		x[5] = PXA[5]; y[5] = PYA[5]; z[5] = PZA[5];
		x[6] = PXA[0]; y[6] = PYA[0]; z[6] = PZA[0];
		x[7] = PXA[3]; y[7] = PYA[3]; z[7] = PZA[3];
		Trapezoid *trap = new Trapezoid(x, y, z, col, gTransparency);
		trap->SetStrNr(gStrnr);
		trap->SetElmId(gElmid);
		trap->SetName(name);
		trap->SetTitle(title);
		trap->Draw();
	}
	
	if ( is_cal && ( gBlockEdges == 0 && gBlockEdgesFill == 0) ) 
		return;
	
	// This is an extension to GROPE: tracks may be colored
	// according to their particle type  (same color as in dedx plot)
	if ( gColorTracks && det == "CTRK" ) {
		gODpbSwitch = 1;
		odpblk_(det.Data(), &gElmid);
		if (gNumberRealOD >30) {
			Int_t ipmaxw = -1;
			Float_t maxw = 0;
			for (Int_t i = 0; i < 5; i++ ) {
				Float_t w = TMath::Abs(gRealOD[i+24]);
				if (w > maxw) {
					maxw = w;
					ipmaxw = i;
				}
			}
			if (maxw > 0.3)
				col = PartCol[ipmaxw];
			else
				col = kColUndef;
			}
		gODpbSwitch = 0;
	}
	
	TPol3D *pl3d; 
	pl3d = new TPol3D(*N, PXA, PYA, PZA);
	pl3d->SetStrNr(gStrnr);
	pl3d->SetElmId(gElmid);
	// this info is displayed if mouse is over a track
	pl3d->SetTitle(title);
	pl3d->SetName(name);
	pl3d->SetLineColor(col);
//	printf(" gPlwidth: %f %f \n",gPlwidth, gThicken);
//	pl3d->SetLineWidth(Width_t(1.));
//	pl3d->SetLineStyle(1);
	if (det == "GEOM" )
		pl3d->SetLineWidth(Width_t(gThickenGeom * gPlwidth));
	else
		pl3d->SetLineWidth(Width_t(gThickenOD * gPlwidth));
	pl3d->SetLineStyle(gPltype);	
	pl3d->Draw();
}
//____________________________________________________

void ppl_(int *N, float *PXA, float *PYA) 
{
	//   POLYLINE
	gCanvas3D->cd();
//	if (*N > 0) return;
	Float_t x[1000], y[1000];
	Int_t np = *N;
	if (np > 1000)
		np = 1000;
	for (Int_t i=0; i < np; i++) {
		Float_t xt = 0.5 * PXA[i];
		Float_t yt = 0.5 * PYA[i];
		x[i] = 20. * (xt - rx0) / (rx1 - rx0);
		y[i] = 20. * (yt - ry0) / (ry1 - ry0);
	}
	ipl_(&np, x, y);
}
//____________________________________________________

void ppm3_(int *N, float *PXA, float *PYA, float *PZA) 
{
//   POLYMARKER 3
//	printf("=================  ppm3 %d %d\n", *N, gPmcolour);
	gCanvas3D->cd();
	TPolyMarker3D *pm3d; 

	pm3d = new TPolyMarker3D(*N);
	for (Int_t i=0; i < *N; i++) {
		pm3d->SetPoint(i, PXA[i], PYA[i], PZA[i]);
	}
	pm3d->SetMarkerColor(gPmcolour);
	pm3d->SetMarkerSize(gMarkSize);
	pm3d->Draw();
}
//____________________________________________________

void ppost_(int WKID, int STRID, float PRIORT) 
{
//   POST STRUCTURE
}
//____________________________________________________

void pprec_(int IL, int *IA, int RL, float *RA, int SL, int *LSTR, char *STR, int MLDR, int ERRIND, int LDR, char *DATREC) 
{
//   PACK DATA RECORD 
}
//____________________________________________________

void pqcf_(int WTYPE, int ERRIND, int NCOLI, int COLA, int NPCI, float *CC) 
{
//   INQUIRE COLOUR FACILITIES
}
//____________________________________________________

void pqcr_(int *WKID, int *COLI, int *CCSBSZ, int *TYPE, int *ERRIND, int *OL, float *CSPEC) 
{
//	printf("=================  pqcr_ %d \n", *COLI);
	*ERRIND = 0;
	*OL = 1;
	CSPEC[0] = 0;
	CSPEC[1] = 0;
	CSPEC[2] = 0;
//   INQUIRE COLOUR REPRESENTATION
}
//____________________________________________________

void pqdlc3_(int WTYPE, int DEVNO, int N, int MLDR, int ERRIND, float DPX, float DPY, float DPZ, int OL, int PET, float *EVOL, int LDR, char *DATREC) 
{
//   INQUIRE DEFAULT LOCATOR DEVICE DATA 3
}
//____________________________________________________

void pqdsp3_(int WTYPE, int ERRIND, int DCUNIT, float DX, float DY, float DZ, int RX, int RY, int RZ) 
{
//   INQUIRE DISPLAY SPACE SIZE 3
}
//____________________________________________________

void pqpost_(int WKID, int N, int ERRIND, int NUMBER, int STRID, float PRIORT) 
{
//   INQUIRE POSTED STRUCTURES
}
//____________________________________________________

void pqstrs_(int * STRSTA) 
{
//   INQUIRE STRUCTURE STATE VALUE
	*STRSTA = 1;
}
//____________________________________________________

void pres_(int N, int *NAMSET) 
{
//   REMOVE NAMES FROM SET
}
//____________________________________________________

void prqlc3_(int WKID, int LCDNR, int STAT, int VIEWI, float PX, float PY, float PZ) 
{
//   REQUEST LOCATOR 3
}
//____________________________________________________

void prqpk_(int WKID, int PKDNR, int IPPD, int STAT, int PPD, int *PP) 
{
//   REQUEST PICK
}
//____________________________________________________

void prst_(int WKID, int COFL) 
{
//   REDRAW ALL STRUCTURES
}
//____________________________________________________

void pschh_(float *CHH) 
{
//   SET CHARACTER HEIGHT
//	printf("=================  pschh_ %f \n", *CHH);
	ischh_(CHH);
	gCharH = *CHH;
}
//____________________________________________________

void pschm_(int WKID, int CHDNR, int MODE, int ESW) 
{
//   SET CHOICE MODE
}
//____________________________________________________

void pschsp_(float CHSP) 
{
//   SET CHARACTER SPACING
}
//____________________________________________________

void pschup_(float CHUX, float CHUY) 
{
//   SET CHARACTER UP VECTOR
}
//____________________________________________________

void pschxp_(float CHXP) 
{
//   SET CHARACTER EXPANSION FACTOR
}
//____________________________________________________

void pscr_(int WKID, int CI, int OL, float *CSPEC) 
{
//   SET COLOUR REPRESENTATION
}
//____________________________________________________

void psdus_(int WKID, int DEFMOD, int MODMOD) 
{
//   SET DISPLAY UPDATE STATE
}
//____________________________________________________

void psedm_(int *EDITMO) 
{
//   SET EDIT MODE
}
//____________________________________________________

void psep_(int EP) 
{
//   SET ELEMENT POINTER
}
//____________________________________________________

void psiasf_(int ATTRID, int ASFVAL) 
{
//   SET INDIVIDUAL ASF
}
//____________________________________________________

void psici_(int *COLI) 
{
//	printf("=================  psici_ %d \n", *COLI);
	gPfacolour = *COLI;
	isfaci_(COLI);
	//   SET INTERIOR COLOUR INDEX
}
//____________________________________________________

void psism_(int ISM) 
{
//   SET INTERIOR SHADING METHOD
}
//____________________________________________________

void psis_(int INTS) 
{
//   SET INTERIOR STYLE
}
//____________________________________________________

void psisi_(int ISTYLI) 
{
//   SET INTERIOR STYLE INDEX
}
//____________________________________________________

void pslcm_(int WKID, int LCDNR, int MODE, int ESW) 
{
//   SET LOCATOR MODE
}
//____________________________________________________

void pslmt3_(float *XFRMT, int CTYPE) 
{
//   SET LOCAL TRANSFORMATION 3
}
//____________________________________________________

void psln_(int * LTYPE) 
{
//   SET LINETYPE
//	printf("=================  psln_ %d \n", *LTYPE);
	gPltype = *LTYPE;

}
//____________________________________________________

void pslwsc_(float * LWIDTH) 
{
//   SET LINEWIDTH SCALE FACTOR
//	printf("=================  pslwsc_ %f \n", *LWIDTH);
	gPlwidth = *LWIDTH;
	if (gPlwidth < 1) 
		gPlwidth = 1.;
}
//____________________________________________________

void psmcv3_(int MCLPOP, int NMCLHS, float *MCLPHS) 
{
//   SET MODELLING CLIPPING VOLUME 3
}
//____________________________________________________

void psmksc_(float *MSZSF) 
{
//   SET MARKER SIZE SCALE FACTOR
//	printf("=================  psmksc_ %f \n", *MSZSF);
	gMarkSize = *MSZSF;
	if (gMarkSize <= 0) 
		gMarkSize = 1;
}
//____________________________________________________

void psmk_(int MTYPE) 
{
//   SET MARKER TYPE
}
//____________________________________________________

void psmlc3_(int WKID, int LCDNR, int VIEWI, float LPX, float LPY, float LPZ) 
{
//   SAMPLE LOCATOR 3
}
//____________________________________________________

void psmst_(int WKID, int STDNR, int LOSTR, char *STR) 
{
//   SAMPLE STRING
}
//____________________________________________________

void pspkft_(int WKID, int PKDNR, int ISN, int *IS, int ESN, int *ES) 
{
//   SET PICK FILTER
}
//____________________________________________________

void pspkid_(int * PKID) 
{
//   SET PICK IDENTIFIER
//	printf("=================  pspkid_ %d \n", *PKID);
	gElmid = *PKID;
}
//____________________________________________________

void psplci_(int *COLI) 
{
//   SET POLYLINE COLOUR INDEX
//	printf("=================  psplci_ %d \n", *COLI);
	gLcolour = *COLI;
	isplci_(COLI);
}
//____________________________________________________

void pspmci_(int *COLI) 
{
//   SET POLYMARKER COLOUR INDEX
//	printf("=================  pspmci_ %d \n", *COLI);
	gPmcolour = *COLI;
}
//____________________________________________________

void psstm_(int WKID, int STDNR, int MODE, int ESW) 
{
//   SET STRING MODE
}
//____________________________________________________

void pstxal_(int *TXALH, int *TXALV) 
{
//   SET TEXT ALIGNMENT
//	printf("=================  pstxal_ %d %d \n", *TXALH, *TXALV);
	gTalign_h = *TXALH;
	gTalign_v = *TXALV;
	istxal_(TXALH, TXALV);
}
//____________________________________________________

void pstxci_(int *COLI) 
{
//   SET TEXT COLOUR INDEX
//	printf("=================  pstxci_ %d \n", *COLI);
	gTcolour = *COLI;
}
//____________________________________________________

void pstxfn_(int *FONT) 
{
//   SET TEXT FONT
//	printf("=================  pstxfn_ %d \n", *FONT);
}
//____________________________________________________

void pstxpr_(int PREC) 
{
//   SET TEXT PRECISION
}
//____________________________________________________

void psvlm_(int WKID, int VLDNR, int MODE, int ESW) 
{
//   SET VALUATOR MODE
}
//____________________________________________________

void psvwi_(int VIEWI) 
{
//   SET VIEW INDEX
}
//____________________________________________________

void psvwr3_(int WKID, int VIEWI, float *VWORMT, float *VWMPMT, float *VWCPLM, int YCLPI, int BCLIPI, int FCLIPI) 
{
//   SET VIEW REPRESENTATION 3
}
//____________________________________________________

void pswkv3_(int WKID, float *WKVP) 
{
//   SET WORKSTATION VIEWPORT 3
}
//____________________________________________________

void pswkw3_(int WKID, float *WKWN) 
{
//   SET WORKSTATION WINDOW 3
}
//____________________________________________________

void ptx3_c_(float *PX, float *PY, float *PZ, float *TDX, float *TDY, float *TDZ, char *CHARS) 
{
//   TEXT 3 
//	printf("  ptx3_ %f %f %f %f %f %f %d %s \n",*PX, *PY, *PZ, *TDX, *TDY, *TDZ, strlen(CHARS), CHARS);
	Size_t ts = 0.015;
	if ( strlen(CHARS) == 1 ) {
		TString temp(CHARS);
		if (temp == "X" || temp == "Y" || temp == "Z")
			ts = 0.00015;
	}
//		return;          // suppress printing X, Y, Z
	TText3D * t3d = new TText3D(*PX, *PY, *PZ, *TDX, *TDY, *TDZ, CHARS);
	t3d->SetTextColor(gTcolour);
	t3d->SetLineWidth(2);
	t3d->SetTextSize(ts);
	t3d->Draw();
}
//____________________________________________________

void ptx_c_(float *PX, float *PY, char* CHARS) 
{
	//   TEXT
	gCanvas3D->cd();
//	printf("====== ptx_c_ %f %f %s \n", *PX, *PY, CHARS);
	float xt = 0.5 * (*PX);
	float yt = 0.5 * (*PY);
	xt = 20. * (xt - rx0) / (rx1 - rx0);
	yt = 20. * (yt - ry0) / (ry1 - ry0);
	float tsize = 0.015;
	ischh_(&tsize);
	int font = 6;
	int prec = 2;
	istxfp_(&font, &prec);
	itx_c_(&xt, &yt, CHARS);
}
//____________________________________________________

void pupast_(int WKID) 
{
//   UNPOST ALL STRUCTURES
}

//____________________________________________________

void puwk_(int WKID, int REGFL) 
{
//   UPDATE WORKSTATION
//	printf("=================  puwk_\n");
	if (gCanvas3D) {
//		printf("=================  puwk_canvas\n");
		gCanvas3D->Modified();
		gCanvas3D->Update();
	}
}
//____________________________________________________

void pwait_(float TOUT, int WKID, int ICL, int IDNR) 
{
//   AWAIT EVENT
}

}
