//
// These routines provide the the functionality of HBOOK, HPLOT and HIGZ
// needed in GROPE by calls to ROOT routines
// This concerns the 2D part of the event display (header, logo, scales),
// the dedx and lego plots
// Only those routines needed for these tasks are implemented
//

#include "TROOT.h"
#include "TVirtualPad.h"
#include "TCanvas.h"
#include "TFrame.h"
#include "TGraph.h"
#include "TH1F.h"
#include "TH2F.h"
#include "THStack.h"
#include "TLatex.h"
#include "TLatex.h"
#include "TList.h"
#include "TMath.h"
#include "TMarker.h"
#include "TPolyLine.h"
#include "TStyle.h"
#include "TSystem.h"
#include "TView3D.h"
#include <map>

using namespace std;
typedef map<Int_t, TH1*> histlist;

extern TView3D * gView3D;
extern TCanvas * gCanvas3D;
extern TCanvas * gCanvasDZ;
extern TLatex * gScaleText;
extern Float_t gScale;
extern Int_t gMainWWidth, gMainWHeight; 
extern Int_t gCmdWWidth, gCmdWHeight;
//TH1F * hist;
	TCanvas * gCanvasDedx = NULL;
	TCanvas * gCanvasLego = NULL;

	Int_t gCurrentHist = 0;
	int gNfuncs = 0;
	
	Int_t gLineColor = 1, gFillColor = 1, gFillStyle = 1001,
	gFillStyleIndex = 3003, gLineWidth = 1, gMarkerSize = 1;
	int gLogx =0, gLogy = 0, gLogz = 0;
	int gTextHalign = 1, gTextValign = 1;
	int gTextColor = 1, gTextFontPrec = 60;
	Double_t gTextSize = 0.03;
	
	int gNormTrans = 1;
	Double_t gHigzWh = 20, gHigzWw = 20;
	Double_t gLegoWh = 20, gLegoWw = 20;
	THStack gStackLego("Grope_Lego", "");
	Color_t gCurrentColor;
	TString gHplTitle;

	histlist hists;

extern "C" {
	
//______________________________________________________________
	
void hbook2_(int *id, const char * nam, 
			int * nbinx, float * xl, float * xu,
			int * nbiny, float * yl, float * yu, float * dy) 
//
// This is needed for the lego plots of tracks and clusters: theta vs. phi
// HBOOKS integer histogram Ids are mapped on ROOTs c++ pointers with the
// help of a map<Int_t, TH1*> histlist from the stl
//
{
	TH2F * h1;
	if ( hists.count(*id) > 0) {
		h1 = (TH2F*)hists[*id];
//		printf("hbook2_%d already existing: %x\n", *id , h1);
//		h1->Print();
		h1->Reset();
		return;
	}
	gCurrentHist = *id;
	TString hn ("h_");
	hn += *id;
	h1 = new TH2F(hn, hn, *nbinx, *xl, *xu, *nbiny, *yl, *yu);
	hists[*id] = h1;
	gStyle->SetOptStat(0);
	gStyle->SetOptTitle(0);
	gNfuncs = 0;
//	printf("hbook2_%d %d %f %f %d %f %f\n", *id,*nbinx,*xl,*xu,*nbiny,*yl,*yu);
};
//______________________________________________________________

void hfill_(int *id, float * x, float * y, float * w)
//
// This is needed for the lego plots of tracks and clusters: theta vs. phi
// Note only filling of 2D hists is needed / implemented
//
{
	if ( hists.count(*id) == 0) {
		printf("hfill_ %d not existing\n", *id );
		return;
	}
	TH2F * h1 = (TH2F*)hists[*id];
	if (h1->GetDimension() != 2) {
		printf("hfill_ %d not 2 dim hist\n", *id );
		return;
	}
	h1->Fill(*x, *y, *w);
}
//______________________________________________________________

void hplego_(int *id, float * theta, float * phi)
{
	if ( hists.count(*id) == 0) {
		printf("hfill_ %d not existing\n", *id );
		return;
	}
	
	TString ci ("c_");
	ci+= *id;
	TCanvas *ca = (TCanvas*)gROOT->GetListOfCanvases()->FindObject(ci);
	
	if ( ca ) {
		ca->cd();
		ca->Clear();
		gCanvasLego = ca;
		gTextSize = 0.03;
	} else {
		gCanvasLego = new TCanvas(ci, ci, 1150, 560, 500, 524);
		gCanvasLego->Draw();
		gTextSize = 0.03;
	}
	gTextHalign = 1;
	gTextValign = 1;
	TH2F * h1 = (TH2F*)hists[*id];
	h1->Draw("LEGO");
}
//______________________________________________________________

void hbook1_(int *id, const char * nam, 
				 int * nbin, float * xl, float * xu, float * dx)
// needed only as histogram for the dedx plot
{
	if ( hists.count(*id) ) {
//		printf("hbook2_%d already existing\n", *id );
		return;
	}
	gCurrentHist = *id;
	TH1F * h1;
	TString hn ("h_");
	hn += *id;
	h1 = new TH1F(hn, hn, *nbin, *xl, *xu);
	hists[*id] = h1;
	gStyle->SetOptStat(0);
	gStyle->SetOptTitle(0);
	gLogx = gLogy = gLogz = 0;
	gNfuncs = 0;
//	printf("hbook1_%d\n", *id);
};
//______________________________________________________________
				 
void hplgiv_(float * xl, float * yl, float * xu, float * yu) 
{
	if ( !gPad ) {
		printf("hplgiv_ gPad = NULL \n");
		return;
	}
	gPad->cd();
	gHigzWw = gPad->GetCanvas()->GetXsizeReal();
	gHigzWh = gPad->GetCanvas()->GetYsizeReal();
	*xl = gHigzWw * gPad->GetCanvas()->GetLeftMargin();
	*yl = gHigzWh * gPad->GetCanvas()->GetBottomMargin();
	*xu = gHigzWw * (1 - gPad->GetCanvas()->GetRightMargin());
	*yu = gHigzWh * (1 - gPad->GetCanvas()->GetTopMargin());
	printf("hplgiv_ %f %f \n", gHigzWw, gHigzWh);
};
//______________________________________________________________

void hminim_(int *id, float * x ) 
{
	TH1 * hh = hists[gCurrentHist];
	if ( hh == NULL ) {
		printf ("hminim_ , no histogram defined \n");
		return;
	}
	hh->SetMinimum(*x);
};
//______________________________________________________________

void hmaxim_(int *id, float * x ) 
{
	TH1 * hh = hists[gCurrentHist];
	if ( hh == NULL ) {
		printf ("hmaxim_ , no histogram defined \n");
		return;
	}
	hh->SetMaximum(*x);
//	printf ("hmaxim_ %f \n", *x);
};
//______________________________________________________________

void hplax_(const char * titleX,const char * titleY) 
{
	TH1 * hh = hists[gCurrentHist];
	if ( hh == NULL ) {
		printf ("hplax , no histogram defined \n");
		return;
	}
	hh->GetXaxis()->SetTitle(titleX);
	hh->GetYaxis()->SetTitle(titleY);
};
//______________________________________________________________

void hplfun_(float * x, float * y, int * n, const char * opt) 
// needed for drawing of the the dedx curves 
{
	TH1F * hh = (TH1F*)hists[gCurrentHist];
	if ( hh == NULL ) {
		printf ("hplfun , no histogram defined \n");
		return;
	}
	TGraph* fun = new TGraph(*n, x, y);
	fun->SetHistogram(hh);
//	printf ("hplfun n, x[30], y[30] %d %f %f \n", *n, x[30], y[30]);
	TString dopt("C");
	if (gNfuncs == 0 )
		dopt += "A";
	Color_t col = gLineColor;
	if (col == 5)
		col = 7;
	fun->SetLineColor(col);
	fun->Draw(dopt);
	gNfuncs ++;
};
//______________________________________________________________

void hpline_(float * x, float * y, int * n, const char * opt) 
{	
	TH1F * hh = (TH1F*)hists[gCurrentHist];
	if ( hh == NULL ) {
		printf ("hpline , no histogram defined \n");
		return;
	}
	TGraph* fun = new TGraph(*n, x, y);
	fun->SetHistogram(hh);
//	printf ("hpline n, x[10], y[10] %d %f %f \n", *n, x[10], y[10]);
	Color_t col = gLineColor;
	if (col == 5)
		col = 7;
	
	fun->SetLineColor(col);
	fun->Draw("C");
};
//______________________________________________________________

void hplot_(int *id,const char * c1,
					const char * c2, int * n) 
{
	TString ci ("C_");
	if (*id == 1113) {
		ci += "Grope_dedx";
	} else  {
		ci+= *id;
	}
	gSystem->ProcessEvents();
	TCanvas *ca = (TCanvas*)gROOT->GetListOfCanvases()->FindObject(ci);
	
	if ( ca ) {
//		printf ("hplot_ use canvas id %d \n", *n);
//	printf("hplot_ 1 %f\n", gPad->GetCanvas()->GetXsizeReal());
		ca->cd();
//	printf("hplot_ 2 %f\n", gPad->GetCanvas()->GetXsizeReal());
		ca->Clear();
		gCanvasDedx = ca;
		gTextSize = 0.03;
	} else {
//		printf ("hplot_ new canvas id %d \n", *n);
		gCanvasDedx = new TCanvas(ci, ci, gMainWWidth+8,gCmdWHeight+30, gCmdWWidth,gCmdWWidth+24);
		gCanvasDedx->Draw();
//		gCanvasDedx->GetFrame()->SetFillColor(1);
//		gCanvasDedx->GetFrame()->SetFillStyle(1001);
		gTextSize = 0.03;
	}
	if ( gLogx )
		gCanvasDedx->SetLogx();
	if ( gLogy )
		gCanvasDedx->SetLogy();
	if ( gLogz )
		gCanvasDedx->SetLogz();
	gCanvasDedx->cd();
	TH1* hh = hists[*id];
	hh->Draw();
	gSystem->ProcessEvents();
//	printf("hplot_ 3 %f\n", gPad->GetCanvas()->GetXsizeReal());
//	gSystem->Sleep(100);
};
//______________________________________________________________

void hplopt_c_(const char * opt, int * n) 
{
//	printf ("hplopt_ n %d opt: %s \n", *n, opt);
	TString sopt(opt);
	if (sopt.Contains("LOGX"))
		gLogx = 1;
	if (sopt.Contains("LOGY"))
		gLogy = 1;
	if (sopt.Contains("LOGX"))
		gLogz = 1;
};
//______________________________________________________________

void hpltab_c_(Int_t * id, Int_t * npars, Float_t * pars, const char * chopt)
// Implement stacking of lego plots using a THStack
{
	static Int_t stack_drawn = 0;
//	printf("hpltab_c_ %d %d %s \n", *id, gCurrentColor, chopt );
	
	if ( hists.count(*id) == 0) {
		printf("hpltab_c_ %d not existing\n", *id );
		return;
	}
	TH2F * h = (TH2F*)hists[*id];
	if (h->GetDimension() != 2) {
		printf("hpltab_c_ %d not 2 dim hist\n", *id );
		return;
	}
	h->SetLineColor(gCurrentColor);
	h->SetFillColor(gCurrentColor);
	if ( stack_drawn > 0) {
		stack_drawn = 0;
		TList *hl = gStackLego.GetHists();
		if ( hl )
			hl->Clear();
	}
	TString opt(chopt);
	gStackLego.Add(h);
	if (opt.Contains("+") )
		return;
	TString dopt("LEGO");
	if (opt.Contains("LEGO2") ) {
		dopt = "LEGO2";
	} else if (opt.Contains("LEGO1") ) {
		dopt = "LEGO1";
	}
	gSystem->ProcessEvents();
	gCanvasLego = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("C_Grope_Lego");
	if ( gCanvasLego ) {
		gCanvasLego->cd();
		gCanvasLego->Clear();
//		printf("hpltab_c_  Clear\n");
		gTextHalign = 1;
		gTextValign = 1;
		gTextSize = 0.03;
		gLogx = gLogy = gLogz = 0;
	} else {
		
		gCanvasLego = new TCanvas("C_Grope_Lego", "Grope_Lego" , 
										  gMainWWidth+8,gCmdWWidth+175, gCmdWWidth,gCmdWWidth+20);
		gCanvasLego->Range(0.,0., gLegoWw, gLegoWh);
		gCanvasLego->Draw();
//		printf("hpltab_c_  Draw canvas\n");
		gTextHalign = 1;
		gTextValign = 1;
		gTextSize = 0.03;
	}
	gPad = gCanvasLego;
	gStackLego.Draw(dopt);
	gSystem->ProcessEvents();
	gCanvasLego->SetPhi(30);
	gCanvasLego->SetTheta(30);
		printf("hpltab_c_ Draw stack %d \n", gStackLego.GetHists()->GetSize());
	stack_drawn = 1;
}	
//______________________________________________________________

void hpltit_c_(const char * title)
{
	gHplTitle = title;
}
//______________________________________________________________

void istxal_(int * ha, int * va)
{
	gTextHalign = *ha;
	gTextValign = *va;
	if (gTextHalign < 1 || gTextHalign > 3)
		gTextHalign = 1;
	if (gTextValign < 1 || gTextValign > 3)
		gTextValign = 1;
	
//	printf ("istxal_ %d %d  \n", gTextHalign, gTextValign);
};
//______________________________________________________________

void istxci_(int * col)
{
	gTextColor = *col;
};
//______________________________________________________________

void iselnt_(int * nt)
{
//	printf("iselnt_ gNormTrans %d\n", *nt);
	gNormTrans = *nt;
};
//______________________________________________________________

void ischh_(Float_t *size)
{
	gTextSize = *size;
//	printf("ischh_ %f\n", *size);
};
//______________________________________________________________

void istxfp_(Int_t *font, Int_t * prec)
{
//	printf("istxfp_ %d %d\n", *font, *prec);
	gTextFontPrec = 10 * TMath::Abs(*font) + 2;
	if (gTextFontPrec > 122)
		gTextFontPrec = 62;
};
//______________________________________________________________

void ifa_(Int_t *n, Float_t * x, Float_t *y)
{
	if ( !gPad ) {
		printf("ifa_ gPad = NULL \n");
		return;
	}
	if (TMath::Abs(x[0]) > 1 || TMath::Abs(y[0]) > 1)
		gNormTrans = 1;
	TPolyLine *pl = new TPolyLine(*n);
	Float_t xf = 1, yf = 1;
	
	if (gNormTrans == 1 ) {
		xf = 1. / gHigzWw;
		yf = 1. / gHigzWh;
	} else {
//		pl->SetNDC(kFALSE);
	}
	Double_t xl = gPad->GetX1();
	Double_t xu = gPad->GetX2();
	Double_t yl = gPad->GetY1();
	Double_t yu = gPad->GetY2();
	Double_t dx = xu - xl;
	Double_t dy = yu - yl;
	Double_t xp, yp;	
//	printf ("ifa_c_Ou %d %d %f %f %f %f %f %f\n",gNormTrans,  *n, dx, xf, x[0], y[0], x[*n-1], y[*n-1]);
	for (Int_t i= 0; i < *n; i++) {
		xp = xf * x[i] * dx + xl;
		yp = yf * y[i] * dy + yl;
		
		if (gLogx) {
			xp = TMath::Power(10,xp);
		}
		if (gLogy) {
			yp = TMath::Power(10,yp);
		}
		
		pl->SetPoint(i, xp ,yp);
	}
//	printf("ifa_ %x %f %d %d\n", (UInt_t)gPad, gPad->GetCanvas()->GetXsizeReal(), gFillStyle, gFillColor);
	pl->SetFillStyle(1001);
	if (gPad == gCanvas3D && gFillColor == 4)
		gFillColor = 66;
	pl->SetFillColor(gFillColor);
	pl->SetLineColor(gFillColor);
	pl->SetLineWidth(2);
	pl->Draw("F");	
};
//______________________________________________________________

void ipl_(Int_t *n, Float_t * x, Float_t *y)
{
	if ( !gPad ) {
		printf("ipl_ gPad = NULL \n");
		return;
	}
	if (TMath::Abs(x[0]) > 1 || TMath::Abs(y[0]) > 1 )
		gNormTrans = 1;
	
//	printf ("ipl_c %d %d %f %f %f %f \n", *n, gNormTrans, x[0], y[0], x[*n-1], y[*n-1]);
	TPolyLine *pl = new TPolyLine(*n);
	Float_t xf = 1, yf = 1;
	
	pl->SetNDC(kTRUE);
	if (gNormTrans == 1 ) {
		xf = 1. / gHigzWw;
		yf = 1. / gHigzWh;
	} 
	for (Int_t i=0; i < *n; i++) {
		pl->SetPoint(i, xf * x[i], yf * y[i]);
	}
	pl->SetLineColor(gLineColor);
	pl->Draw();
};
//______________________________________________________________

void itx_c_(float * x, float * y, const char * c) 
{
	if (TMath::Abs(x[0]) > 1 || TMath::Abs(y[0])> 1)
		gNormTrans = 1;
	Double_t xt =  (Double_t)*x;
	Double_t yt = (Double_t)*y;
//	gPad->cd();
	Double_t xf = 1, yf = 1;
	TLatex * l;
	if (gNormTrans == 1 ) {
		xf = 1. / gHigzWw;
		yf = 1. / gHigzWh;
	} 
	if (gPad == gCanvasDedx) {
		// heuristic correction of alignment of labels in dedx plot
		Double_t cfac = 0.91;
		Double_t xmarg = 2.;
		Double_t xtNDC = xf * ( xmarg + (xt - xmarg) * cfac );
		Double_t ytNDC = yt * yf;
		// convert NDC coordinates to user in order that labels remain
		// aligned to ellipses of dedx during zooming
		Double_t xuser = gPad->GetX1() + (gPad->GetX2() - gPad->GetX1()) * xtNDC;
		xuser = TMath::Power(10, xuser) ;
		Double_t yuser = gPad->GetY1() + (gPad->GetY2() - gPad->GetY1()) * ytNDC;
		yuser = TMath::Power(10, yuser) ;
//		printf ("itx_c %f %f %f %f %s\n", xtNDC, ytNDC, xuser, yuser, c);
		l = new TLatex(xuser, yuser, c);
		l->SetNDC(kFALSE);
//		l = new TLatex(xtNDC, ytNDC, c);
//		l->SetNDC(kTRUE);
	} else {
		l = new TLatex(xf*xt, yf*yt, c);
		l->SetNDC(kTRUE);
	}
	
	// initialize the quick and dirty correction of the scale value
	// at bottom of event display |--------|
	//                              100.cm.
	TString text(c);
	Int_t ind = text.Index("cm.");
	if (ind > 1) {
		text.Resize(ind);
		if (text.IsFloat()) {
			gScaleText = l;
			gScale = text.Atof();
		}
	}	
	Float_t ts = gTextSize;
	if (ts > 0.1) {
		if (gPad == gCanvasDZ)
			ts = 0.015;
		else
			ts = 0.03;
	}
	l->SetTextSize(ts);
//	printf ("itx_c %f %f %f %s \n", *x, *y, ts,  c);
	Color_t col = gTextColor;
	if (col == 5)
		col = 7;
	l->SetTextColor(col);
	l->SetTextAlign(gTextValign + 10* gTextHalign);
	l->SetTextFont(gTextFontPrec);
	l->Draw();
};
//______________________________________________________________

void isfaci_(Int_t *col)
{
	gFillColor = *col;
//	if (gFillColor == 4)   //cyan
//		gFillColor = 7;
};
//______________________________________________________________

void isfais_(Int_t *style)
{
	gFillStyle = *style;
//	printf("isfais_ %d\n", *style);
};
//______________________________________________________________

void isfasi_(Int_t *style)
{
//	printf("isfasi_ %d\n", *style);
	gFillStyleIndex = *style;
};
//______________________________________________________________

void isplci_(int *col)
{
	gLineColor = *col;
};
//______________________________________________________________

void iscr_(Int_t *iwk, Int_t *colind, Float_t *red, Float_t* green, Float_t *blue)
{
	gCurrentColor = *colind;
}
//______________________________________________________________

void iclrwk_(Int_t *iwk, Int_t *icoflg)
{
	if (gROOT->GetListOfCanvases()->FindObject("C_Grope_DzDisp") ) {
//		printf("iclrwk_ %d \n", *iwk);
		if ( gCanvasDZ ) {
			gCanvasDZ->Clear();
			gCanvasDZ->cd();
		}
		gTextFontPrec = 62;
	}
}
//______________________________________________________________
void iopwk_(Int_t *id1 ,Int_t *id2, Int_t *id3)
{
//	printf("iopwk_ %d %d %d\n", *id1, *id2, *id3 );
};
//______________________________________________________________

void iacwk_(Int_t *id1)
{
//	printf("iacwk_ %d \n", *id1);
};
//______________________________________________________________

void idawk_(Int_t *id1)
{
// 	gSystem->ProcessEvents();
// 	gSystem->Sleep(1000);
// 	gSystem->ProcessEvents();
// 	printf("idawk_ %d \n", *id1);
// 	if (gPad)
// 		printf("ifa_ %f\n", gPad->GetCanvas()->GetXsizeReal());
	if (gROOT->GetListOfCanvases()->FindObject("C_Grope_dedx") ) {
		gCanvasDedx->Modified();
		gCanvasDedx->Update();
	}
	if (gROOT->GetListOfCanvases()->FindObject("C_Grope_Lego") ) {
		gCanvasLego->Modified();
		gCanvasLego->Update();
	}
	gSystem->ProcessEvents();
	gLogx = gLogy = gLogz = 0;
};
//_____________________________________________________________________

void iuwk_(Int_t *id1)
{
	if ( !gROOT->GetListOfCanvases()->FindObject("C_Grope_DzDisp") )
		return;
//	printf("iuwk_ %d \n", *id1);
	if (gPad == gCanvasDZ) {
		gCanvasDZ->Modified();
		gCanvasDZ->Update();
	}
};
//_____________________________________________________________________

void iclwk_(Int_t *id1)
{
//	printf("iclwk_ %d \n", *id1);
};
//_____________________________________________________________________

void irqlc_c_(Int_t *istat, float *x, float *y)
{
	if ( !gROOT->GetListOfCanvases()->FindObject("C_Grope_DzDisp") )
		return;
	if (gPad != gCanvasDZ) 
		return;
	gCanvasDZ->Modified();
	gCanvasDZ->Update();
	TMarker *mark = (TMarker*)gPad->WaitPrimitive("TMarker");
	if (mark == NULL) {
//		printf(" irqlc_c_ mark == NULL\n");
		*x = 0.85;     // position of "Quit" DZDISP
		*y = 0.39;
		*istat = 0;
	} else {
		*x = mark->GetX();
		*y = mark->GetY();
		*istat = 1;
		delete mark;
	}
};

	void hplnul_() {};
	void hdelet_() {};
	void hplset_() {};
	void hplsiz_() {};
   void hplsof_() {};
   void hplzon_() {};
   void igmeta_() {};
   void igrng_() {};
   void igset_() {};
   void ipm_() {};
   void irqst_() {};
   void isln_() {};
   void islwsc_() {};
   void ismk_() {};
   void ispmci_() {};
   void isvp_() {};
   void iswn_() {};
	
   void zbrdef_() {};
	void csaddr_() {};
	void cscom_() {};
	void csexec_() {};
	void csext_() {};
	void csinit_() {};
	void csjcal_() {};
	void csjsub_() {};
	void cspaus_() {};
	void cssetl_() {};
}
