#include "TApplication.h"
#include "TRint.h"
#include "TROOT.h"
#include "TSystem.h"
#include "TCanvas.h"
#include "TGClient.h"
#include "TGLSAViewer.h"
#include "TGLOrthoCamera.h"
#include "TGLCameraOverlay.h"
#include "TView3D.h"
#include "TLatex.h"
#include "TRegexp.h"
#include "ropecmds.h"

#include "TString.h"
#include <iostream>
#include <fstream>
#include <map>
#include <string>

namespace std {} using namespace std;

// store documentation of OD banks in a map
typedef map<string, string> bankdoc;
bankdoc odbanks;

RopeCmds * gRC;

TCanvas * gCanvas3D = NULL;
TCanvas * gCanvasCmd = NULL;
TCanvas * gCanvasDZ = NULL;
TView3D * gView3D;

extern "C" { void view3d_(); }
extern "C" { Int_t bankdoc2map(); }

TGLViewer * gGLViewer = NULL;
// to store output from OD print block

Int_t gODpbSwitch = 0;
static const Int_t gMaxIntOD = 500;
static const Int_t gMaxRealOD = 500;

TString gBlockName;
Int_t   gBlockNumber;
Int_t   gBlockSeq;
Int_t   gIntOD[gMaxIntOD];
char    gIntName[gMaxIntOD][5];
Int_t   gNumberIntOD;
Float_t gRealOD[gMaxRealOD];
char    gRealName[gMaxRealOD][5];
Int_t   gNumberRealOD;

// window sizes and ranges

UInt_t gScreenWidth, gScreenHeight;
Int_t gMainWWidth = 950, gMainWHeight = gMainWWidth+24;
Int_t gCmdWWidth=450, gCmdWHeight = 110;
Double_t rx0 = -450, rx1 = 450, ry0 = -450, ry1 = 450, rz0 = -450, rz1 = 450;

// pointers to command buttons

TButton *gB_xyView = NULL , *gB_yzView = NULL, *gB_xzView = NULL,
*gB_SideView = NULL, *gB_HitsOnOff = NULL, *gB_ThrustOnOff = NULL, 
*gB_JetsOnOff = NULL,  *gB_ApplyPCut = NULL, *gB_ColorCT = NULL,
*gB_Dzdisp, *gB_Dzddiv, *gB_Options = NULL, *gB_Curr = NULL; 

extern "C" {

void view3d_()
{
	// create view with axis
	Double_t rmin[3], rmax[3];
	rmin[0] = rx0;
	rmin[1] = ry0;
	rmin[2] = rz0;
	rmax[0] = rx1;
	rmax[1] = ry1;
	rmax[2] = rz1;
	gCanvas3D->cd();
	gView3D = new TView3D(1, rmin, rmax);
//	gView3D->ShowAxis();
}
//______________________________________________________________________

void openroot_() 
{
/////////////////////////////////////////////////////////////////////
//                                                                 //	
//   Open ROOT and the main canvas for 3D graphics                 //
//   Prepare a simple command interface with TButtons              //
//   Called by kuwham in kuip2root.cxx                             //
//                                                                 //
/////////////////////////////////////////////////////////////////////

	Int_t screen_x, screen_y;

	Int_t argc = 0;
	char * argv[10];
	static Int_t init_done = 0;
	if ( init_done ) 
		return;
	init_done = 1;
	TRint *theApp = new TRint("App", &argc, argv);
	if ( gClient && gClient->GetRoot() ) {
		gVirtualX->GetWindowSize(gClient->GetRoot()->GetId(),
									 screen_x, screen_y, gScreenWidth,gScreenHeight );
	} else {
		gScreenWidth = 1600;
		gScreenHeight = 1200;
	}
	printf(">>>>>>>>>>>>>>>>> gClient gScreenWidth %x %d\n", (UInt_t)gClient , gScreenWidth);
	
	if (gCanvas3D == NULL) {
		gCanvas3D = new TCanvas( "OPAL3D", "OPAL3D", 0, 0, gMainWWidth, gMainWHeight);
		gCanvas3D->Range(rx0, ry0, rx1, ry1);
		gCanvas3D->Draw();
		gCanvas3D->ToggleEventStatus();
		view3d_();
		gCanvas3D->Modified();
		gCanvas3D->Update();
		gSystem->ProcessEvents();
	} 
	
	gRC= new RopeCmds();
	
	if (gCanvasCmd == NULL) {
		gCanvasCmd = new TCanvas( "GropeCmd", "Grope Commands", 
										  gMainWWidth+8, 0, gCmdWWidth, gCmdWHeight);
		gCanvasCmd->Draw(); 
		TButton *b;
		Double_t x0 = 0, y0 = 1, x, y, dx = 0.1666, dy = 0.25;
		x = x0;
		y = y0-dy;
		b = new TButton("AnyEvent","gRC->anyevent()", x, y, x+dx, y+dy);
		b->SetToolTipText("Select input file with multi hadron events");
		b->Draw();
		x+=dx;
		b = new TButton("Hadron","gRC->hadron()", x, y, x+dx, y+dy);
		b->SetToolTipText("Select input file with multi hadron events");
		b->Draw();
		x+=dx;
		b = new TButton("Electron","gRC->electron()", x, y, x+dx, y+dy);
		b->SetToolTipText("Select input file with muon events");
		b->Draw();
		x+=dx;
		b = new TButton("Muon","gRC->muon()", x, y, x+dx, y+dy);
		b->SetToolTipText("Select input file with muon events");
		b->Draw();
		x+=dx;
		b = new TButton("Tau","gRC->tau()", x, y, x+dx, y+dy);
		b->SetToolTipText("Select input file with tau events");
		b->Draw();
		x+=dx;
		b = new TButton("Cosmic","gRC->cosmic()", x, y, x+dx, y+dy);
		b->SetToolTipText("Select input file with cosmic ray events");
		b->Draw();

		// new row
		x = x0;
		y -= dy;
		b = new TButton("OpenGL","gRC->viewOpenGL()",x, y, x+dx, y+dy);
		b->SetToolTipText("View with OpenGL");
		b->Draw();
		x+=dx;
		b = new TButton("Next","gRC->trig()",x, y, x+dx, y+dy);
		b->SetToolTipText("Get and display next event");
		b->Draw();
		x+=dx;
		b = new TButton("Previous","gRC->previous()",x, y, x+dx, y+dy);
		b->SetToolTipText("Get and display previous event");
		b->Draw();
		x+=dx;
		b = new TButton("Lego","gRC->lego()",x, y, x+dx, y+dy);
		b->SetToolTipText("Show energy distribution Cos(theta) x Phi");
		b->Draw();
		x+=dx;
		b = new TButton("Dedx","gRC->dedx()", x, y, x+dx, y+dy);
		b->SetToolTipText("Show dedx plot");
		b->Draw();
		x+=dx;
		b = new TButton("OD Summ","gRC->printodterse()", x, y, x+dx, y+dy);
		b->SetToolTipText("Print a short summary of all tracks");
		b->Draw();

		// new row
		x = x0;
		y -= dy;
		b = new TButton("Zoom","gRC->zoom2()",x, y, x+dx, y+dy);
		b->SetToolTipText("Zoom factor 2");
		b->Draw();
		x+=dx;
		b = new TButton("UnZoom","gRC->unzoom2()",x, y, x+dx, y+dy);
		b->SetToolTipText("Un Zoom factor 2");
		b->Draw();
		x+=dx;
		b = new TButton("xyView","gRC->xyview()",x, y, x+dx, y+dy);
		b->SetToolTipText("View in beam direction");
		b->Draw();
		gB_xyView = b;
		x+=dx;
		b = new TButton("yzView","gRC->yzview()",x, y, x+dx, y+dy);
		b->SetToolTipText("View from side");
		b->Draw();
		gB_yzView = b;
		x+=dx;
		b = new TButton("xzView","gRC->xzview()",x, y, x+dx, y+dy);
		b->SetToolTipText("View from top");
		b->Draw();
		gB_xzView = b;
		x+=dx;
		b = new TButton("SideView","gRC->sideview()",x, y, x+dx, y+dy);
		b->SetToolTipText("View Plane in thrust axis");
		b->Draw();
		gB_SideView = b;
		x = x0;
		y -= dy;
		b = new TButton("DzDisp","gRC->dzdisp()",x, y, x+dx, y+dy);
		b->SetToolTipText("Display ZEBRA bank structure");
		b->Draw();
		gB_Dzdisp = b;
		x += dx;
		b = new TButton("DzDDiv","gRC->dzddiv()",x, y, x+dx, y+dy);
		b->SetToolTipText("Display ZEBRA divisions");
		b->Draw();
		gB_Dzddiv= b;
		x += dx;
		
		b = new TButton("More Cmds","gRC->Actions()",x, y, x+dx, y+dy);
		b->SetToolTipText("Show extended set of actions");
		b->Draw();
		gB_Options = b;
		x += dx;
		
		b = new TButton("Help","gRC->PrintHelp()",x, y, x+dx, y+dy);
		b->SetToolTipText("Print a short help");
		b->Draw();
		gB_Options = b;
		x += dx;
		
		b = new TButton("Options","gRC->DisplayOptions()",x, y, x+dx, y+dy);
		b->SetToolTipText("Options, what to show etc.");
		b->Draw();
		gB_Options = b;
		x += dx;
		
		b = new TButton("Quit","gSystem->Exit(0)",x, y, x+dx, y+dy);
		b->SetToolTipText("Terminate Grope");
		b->Draw();
		gSystem->ProcessEvents();
		TRootCanvas *ci = (TRootCanvas *)gCanvasCmd->GetCanvasImp();
		if ( ci ) 
		TQObject::Connect(ci, "CloseWindow()",
							"RopeCmds", gRC, "HandleCloseCmdWindow()");
		else
			printf("openroot: No frame for Cmd canvas!!\n");
		
		Int_t ndoc = bankdoc2map();
		if ( ndoc  <= 0 ) {
			cout << "No bank doc available" << endl;
		} else {
			cout << "Found doc for " << ndoc << " values" << endl;
		}
		if ( !gSystem->AccessPathName("grope_defaults.C") ) {
			gROOT->ProcessLine(".x grope_defaults.C");
		}
		gB_Curr = gB_xyView;
		
		gRC->BuildMenu();
	} 
	theApp->Run(kTRUE);
	//	popwk_();
}
//________________________________________________________________________

Int_t bankdoc2map()
{
/////////////////////////////////////////////////////////////////////
//                                                                 //
// This routines fills the associative array odbanks with the name //
// of a variable as key and the description as value               //
// It is used to print OD data blocks with their description       //
//                                                                 //
/////////////////////////////////////////////////////////////////////
	TString infile = "/data/od_bank_desc.txt";
	infile.Prepend(gSystem->Getenv("GROPE"));
	ifstream inf;
	inf.open(infile, ios::in);
	if (!inf.good()) {
		cerr	<< "Error: "<< gSystem->GetError() << " - " << infile<< endl;
		return -1;
	}
//	bankdoc odbanks;
	TString line;
	TString block;
	TString name;
	TString seqnr;
	
	TRegexp space(" ");
	
	Bool_t indata =kFALSE;
	Int_t ndoc = 0;
//	TMap * oddoc = new TMap(1000);
//	oddoc->SetName("oddoc");
	while ( 1 ) {
		line.ReadLine(inf);
		if (inf.eof()) break;
		line=line.Strip(TString::kBoth);
		
		if ( line.BeginsWith("*B..") ) {
			block = line(4,4);
			while(block.Index(space) >= 0)
				block(space) = "_";
			cout << "Block: " << block << " *****************************" << endl;
			continue;
		}
		if ( line.BeginsWith("*B/DATA") ) {
			indata = kFALSE;
			continue;
		}
		if ( line.BeginsWith("*B.DATA") ) {
			indata = kTRUE;
			continue;
		}
		if ( !indata ) {
			continue;
		}
		// chop off *B.
		line = line(3,line.Length());
		Ssiz_t pos = 0, slen;
		Bool_t ok;
		const Char_t * del = " ";
		ok = line.Tokenize(seqnr, pos, del);
		if ( ok ) {
			if ( !seqnr.IsDigit() ) 
				continue;
			slen = seqnr.Length();
//			cout << "<<<<<" << slen << " " << line << endl;
			line = line(slen+1, line.Length());
			line = line.Strip(TString::kBoth);
			pos = 0;
			ok = line.Tokenize(name, pos, del);
			if ( ok ) {
				slen = name.Length();
				line = line(slen+1, line.Length());
				line=line.Strip(TString::kBoth);
				name = name(2,name.Length());
				while (name.Length() < 4) 
					name.Append("_");
				name.Prepend(block);
				string key = name.Data();
				string value = line.Data();
//				oddoc->Add(key, value);
				odbanks[key] = value;
				ndoc ++;
//				cout << "Name: " << name << " Doc: " << line << endl;
			}
		}
	}
//	string b("SIW_ITDW");
//	cout << b << " " <<  odbanks[b] << endl;
	return ndoc;
}
//______________________________________________________________________

void grbrow_()
{
	printf("Dummy grbrow_ called");
}
//______________________________________________________________________

void odpbkx_c_(Int_t *idat,Int_t *nimov,char *cifor,char *cinam,
				  Float_t *rdat, Int_t *nrmov,char *crfor,char *crnam)
{
///////////////////////////////////////////////////////////////////////
//                                                                   //
// This routine replaces the FORTRAN version of ROPE OD,             //
// its call is triggered by odpblk                                   //
// Its purpose is to retrieve the OD data by their names to avoid    //
// the need of the  knowledge of their offsets in the data blocks    //
// Since this routine is called from ROPE (FORTRAN) we need   global //
// variables to store data and steer its behaviour                   //
// gODpbSwitch == 1:                                                 //
// only the global arrays gIntOD etc are filled, no printing is done //
// gODpbSwitch == 2:                                                 //
// print the values one per line with their description if available //
//                                                                   //
///////////////////////////////////////////////////////////////////////
	TString scifor(cifor);
	TString scinam(cinam);
	TString scrfor(crfor);
	TString scrnam(crnam);
	TString snam;
	// gBlockNumber == 0: Print all blocks, therefore need internal counter gBlockSeq
	if ( gBlockNumber == 0 )
		gBlockSeq++;
	else 
		gBlockSeq = gBlockNumber;
	Int_t ip = 0;
	if (gODpbSwitch == 1) {
		// copy only
		if (*nimov > gMaxIntOD) {
			printf("Too many Int_t words %d\n", *nimov);
			return;
		}
		ip = 0;
		for (Int_t i = 0; i < *nimov; i++) {
			gIntOD[i] = idat[i];
			snam = scinam(ip, 4);
			
//			char c[5] = { snam.Data() };
			for (Int_t k =0; k<5; k++)
				gIntName[i][k] = snam(k);
//			cout << "gIntName, snam " << gIntName[i] << " " << snam << endl;
			ip += 4;
		}
		gNumberIntOD = *nimov;
		if (*nrmov > gMaxRealOD) {
			printf("Too many real words %d\n", *nrmov);
			return;
		}
		ip = 0;
		for (Int_t i = 0; i < *nrmov; i++) {
			gRealOD[i] = rdat[i];
			snam = scrnam(ip, 4);
			for (Int_t k =0; k<5; k++)
				gRealName[i][k] = snam(k);
			ip += 4;
		}
		gNumberRealOD = *nrmov;
		TString dummy = gRealName[2];
//		cout << "gRealName[2] " << dummy << endl;
		return;
	}
	if ( gODpbSwitch == 2 ) {
		if ( odbanks.size() <=0 ) {
			cout << "No bank doc stored" << endl;
			return;
		}
	}
	TString sfor, format, fullname;
	TRegexp ic("I");
	TRegexp zc("Z");
	TRegexp space(" ");
	ip = 0;
	Int_t valuesperline = 5;
	if (gODpbSwitch == 2) {
		valuesperline = 1;
	}
	cout <<endl << "Integer data for: " << gBlockName << " track/hit: " << gBlockSeq << endl<< endl;
	for (Int_t i = 0; i < *nimov; i++) {
		sfor = scifor(ip, 4);
		snam = scinam(ip, 4);
		sfor = sfor.Strip(TString::kBoth);
//		printf ("sfor: %s \n", sfor.Data());
		if (sfor.BeginsWith("I")) {
			sfor=sfor(1, sfor.Length());
			sfor.Append("d");
		}
		if (sfor.BeginsWith("Z")) {
			sfor=sfor(1, sfor.Length());
			sfor.Append("x");
		}
		format = "%s %";
		format += sfor;
		format += " ";
		
		printf(format.Data(), snam.Data(), idat[i]);
		if ( gODpbSwitch == 2 ) {
			fullname = gBlockName;
			while (fullname.Length() < 4) 
				fullname.Append("_");
			fullname += snam;
			while (fullname.Index(space) >=0) 
				fullname(space) = "_";
			string ind(fullname.Data());
			cout << " " << odbanks[ind];
		}
		if ( (i+1)%valuesperline == 0)
			printf("\n");
		ip += 4;
	}
	printf("\n");
	cout << "Real data for: " << gBlockName << " track/hit: " << gBlockSeq << endl<< endl;
	ip = 0;
	for (Int_t i = 0; i < *nrmov; i++) {
		sfor = scrfor(ip, 4);
		snam = scrnam(ip, 4);
		sfor = sfor.Strip(TString::kBoth);
//		printf ("sfor: %s \n", sfor.Data());
		if (sfor.BeginsWith("F")) {
			sfor=sfor(1, sfor.Length());
			sfor.Append("f");
		}
		if (sfor.BeginsWith("E")) {
			sfor=sfor(1, sfor.Length());
			sfor.Append("g");
		}
		format = "%s %";
		format += sfor;
		format += " ";
		
		printf(format.Data(), snam.Data(), rdat[i]);
		if ( gODpbSwitch == 2 ) {
			fullname = gBlockName;
			while (fullname.Length() < 4) 
				fullname.Append("_");
			fullname += snam;
			while (fullname.Index(space) >=0) 
				fullname(space) = "_";
			string ind(fullname.Data());
			cout << " " << odbanks[ind];
		}
		if ( (i+1)%valuesperline == 0)
			printf("\n");
		ip += 4;
	}
	printf("\n");
}	
}
//

