#include "TString.h"
#include <iostream>
#include <fstream>

/*
These routines replace the KUIP package in GROPE
Only those entry points are defined which are referenced in GROPE
Only those needed to interact with GROPE are implemented

Hence: These routines cannot be used elsewhere to replace KUIP
       However they may give a hint how to do it in other applications

Essentially only "Commands" are implemented 

Example GOTO in rocdf.cdf

>Command GOTO
>Parameters
IEVGET 'Get event#'                  I  D=0
+
IRUGET 'In run#'                     I  D=0
CWHAT 'In data/MC/hotline/current' C D='CURRENT' R='DATA,MCARLO,HOTLINE,CURRENT'
>Guidance
With option DATA or MCARLO:
Get event IEVGET of run IRUGET for data or MC from shift-server
>Action ROGOKU

The implementation of the replacement looks as follows:

void RopeCmds::getevent(Int_t ev, Int_t run)
{
	// Get event by number and run
	
	cmd = "GOTO";
	n_args = 3;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	argi[0] = ev;
	argi[1] = run;
	*(argc[0]) = "CURRENT";
	printf("getevent: %d %d\n", ev, run);
	rogoku_();
}

I.e. the command name "GOTO" and the argument values are put
into global arrays one for each type (int, float, etc.)
The FORTRAN routine ROGOKU (rogoku_) reads the command name (kupatl)
and then the expected argument values (kugeti_, kugetc_ etc.)

A root macro kuipcmdec.C is provided to generate the c++ code 
from KUIPs CDF to a large extent

Thats it.

*/
namespace std {} using namespace std;

// globals for kuip calls

TString cmd;
Int_t n_args;

Int_t argi[10];
Int_t argi_count;

TString * argc[10];
Int_t argc_count;

Float_t argf[10];
Int_t argf_count;

extern "C" void openroot_();

extern "C" {
	
void kuinim_(const char * name = NULL)
{
	printf(">>>>>>>>>>>>>>>>> kuinim_\n");
}
//______________________________________________________________________

void kuinit_(Int_t * n)
{
}
//______________________________________________________________________

void kuexec_(const char * name = NULL)
{
}
//______________________________________________________________________

void kuwham_(const char * name = NULL)
{
	printf(">>>>>>>>>>>>>>>>> kuwham_\n");
	
	openroot_();
}
void kuwhat_()
{
	kuwham_("Grope");
}
void kuwhag_()
{
	kuwham_("Grope");
}
//__________________________________________________________

void kuidfm_()
{
}	
void kucmd_(char * arg1, char * arg2, char * arg3)
{
}	
void kuproc_(char * prompt, char * value, Int_t * len)
{
}	
void kupros_(char * prompt, char * value, Int_t * len)
{
}	
void kuproi_(char * prompt, Int_t * value)
{
}	
void kupror_(char * prompt, Float_t * value)
{
}	
void kupar_(char * arg1, char * arg2, char * arg3, char * arg4, char * arg)
{
}	
void kuact_(char * arg1, void * act)
{
}	
void kuguid_(char * arg1, char * arg2, Int_t * n, char * arg3)
{
}	
void kupval_(char * arg1, char * arg2, Int_t * n, Float_t * real, char * arg3, char * arg4)
{
}	

int kunpar_()
{
	return n_args;
}

void kupatl_(char * arg, Int_t * n)
{
// pass the command name to FORTRAN caller

//	printf("kupatl_ %s\n", cmd.Data());
	Int_t len = cmd.Length();
	Int_t len4 = (cmd.Length()+3) / 4 * 4;
	for (Int_t i = 0; i < len4-len; i++)
		cmd.Append(" ");
	for (Int_t i=0; i < len4; i++) {
		arg[i] = cmd.Data()[i];
	}
	*n = n_args;
}

void kugeti_(Int_t * arg)
{
// pass an integer to FORTRAN caller
	*arg = argi[argi_count];
	argi_count++;
}

void kugetr_(Float_t * arg)
{
// pass a real value to FORTRAN caller

	*arg = argf[argf_count];
	argf_count++;
}

void kugetc_(char * arg, Int_t * n)
{
// pass a character string to FORTRAN caller

	(argc[argc_count])->ToUpper();
	Int_t len =  (argc[argc_count])->Length();
	Int_t len4 =  (len +3) / 4 * 4;
	for (Int_t i=0; i< len4-len ; i++) 
		argc[argc_count]->Append(" ");
	for (Int_t i=0; i < len4 ; i++) {
		arg[i] = (argc[argc_count]->Data())[i];
	}	
	*n = len;
	argc_count++;
}

void kugets_c_(char * arg, Int_t * n)
{
// pass a character string to FORTRAN caller

	Int_t len =  (argc[argc_count])->Length();
	Int_t len4 =  (len +3) / 4 * 4;
	for (Int_t i=0; i< len4-len ; i++) 
		argc[argc_count]->Append(" ");
	for (Int_t i=0; i < len4 ; i++) {
		arg[i] = (argc[argc_count]->Data())[i];
	}	
	*n = len;
	argc_count++;
}
}
