#include "TApplication.h"
#include "TRint.h"
#include "TROOT.h"
#include "TSystem.h"
#include "TCanvas.h"
#include "TGLSAViewer.h"
#include "TGLOrthoCamera.h"
#include "TGLCameraOverlay.h"
#include "TGLLogicalShape.h"
#include "TView3D.h"
#include "TLatex.h"
#include "TMarker.h"
#include "TRegexp.h"
#include "TString.h"
#include "TRootHelpDialog.h"

#include "ropecmds.h"

#include <iostream>
#include <fstream>
#include <map>
#include <string>

namespace std {} using namespace std;

// globals for kuip calls

extern TString cmd;
extern Int_t n_args;

extern Int_t argi[10];
extern Int_t argi_count;

extern TString * argc[10];
extern Int_t argc_count;

extern Float_t argf[10];
extern Int_t argf_count;


// to store output from OD print block

extern Int_t gODpbSwitch;
static const Int_t gMaxIntOD = 500;
static const Int_t gMaxRealOD= 500;

extern TString gBlockName;
extern Int_t   gBlockNumber;
extern Int_t   gBlockSeq;
extern Int_t   gIntOD[gMaxIntOD];
extern char    gIntName[gMaxIntOD][5];
extern Int_t   gNumberIntOD;
extern Float_t gRealOD[gMaxRealOD];
extern char    gRealName[gMaxRealOD][5];
extern Int_t   gNumberRealOD;

extern TLatex *  gScaleText;
extern Float_t   gScale;

extern TCanvas * gCanvasCmd;
extern TCanvas * gCanvasDZ;
extern Int_t   gNormTrans;
extern Int_t gMainWWidth, gMainWHeight;
extern Int_t gCmdWWidth, gCmdWHeight;
extern UInt_t gScreenWidth, gScreenHeight;
// pointers to command buttons

extern TButton *gB_xyView, *gB_yzView, *gB_xzView, *gB_SideView, 
*gB_HitsOnOff, *gB_ThrustOnOff, *gB_JetsOnOff, 
*gB_ApplyPCut, *gB_ColorCT, *gB_Curr; 

Int_t gColorTracks;
Float_t gThickenOD = 1;      // apply factor to linewidth tracks, blocks 
Float_t gThickenGeom = 1;    // apply factor to linewidth to GEOM
Int_t gIniHigz = 0;

static const Int_t kNColDet = 6;
Int_t gColDet[kNColDet] = {7, 2, 2, 4, 2, 6};
const char * gDetName[kNColDet] = {"CTRK", "MUON","VRTX", "ECAL", "TOF", "HCAL"};
static const Int_t kNLineTypes = 4;
const char * gLineTypes[kNLineTypes] = {"SOLID", "DOT", "DASH", "DASHDOT"};
Int_t gBlockEdges      = 0;     // 1 draw edges only, 2 fill only, 3 both
Int_t gBlockFill       = 0;     // 1 draw edges only, 2 fill only, 3 both
Int_t gBlockEdgesFill  = 1;
Int_t gTransparency    = 50;    // 100 transparent 0 opaque

ClassImp(RopeCmds)
//__________________________________________________________
	
RopeCmds::RopeCmds() 
{
////////////////////////////////////////////////////////////////////////////
//                                                                        //
// This class implements an interface to GROPE a la ROOT                  //
//
// For interaction with GROPE it uses the routines of kuip2root which     //
// replace the KUIP package of the CERNLIB i.e. routines here fill the    //
// argument arrays (commands, strings, integer, float) which are read by  //
// the KUIP routines kapatl, kugeti, etc.                                 //
// All methods are accessible from the commandline via a global pointer   //
// gRC: e.g. gRC->previous();                                             //
// Methods starting with upper case are normally not called by the user   //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

	fDialog = NULL;
	fDialogCmd = NULL;
	for (Int_t i=0; i < 10; i++) {
		argc[i] = new TString();
	}
	fHitsOn    = 0;
	fHitsOff   = 1;
	fGeomOn = 0;
	fGeomOff = 0;
	fThrustOnOff = 0;
	fJetsOnOff   = 0;
	fPCutOnOff = 0;
	fPCut = 0.2;
	fDetHits = "";
	fDetGeom   = "CZME";
	fPCutScope = "ALL";
	fDetTypeOD = "GNRL";
	gThickenOD = 1.;
	gThickenGeom = 1.;
	fThicken = 1.;
	fThickenScope = "TRACKS";
	fODComplValue = 1;
	fBlockNrOD = 0;
	fWithDocuOD = 0;
	fCurrentRun = -1;
	fPreviousRun = -1;
	fCurrentEvent = -1;
	fPreviousEvent = -1;
	fTotalEvents = 0;
	fEventNr = 0;
	fRunNr = 0;
	fViewDir = kViewXY;
	gGLViewer = NULL;
	fYZCameraAngle = 0;
	fZoom = 0.78;
//	fZoom = 1.5;
	fCenter[0] = 0;
	fCenter[1] = 0;
	fCenter[2] = 0;
	fMuonFile     = gSystem->Getenv("MUONFILE");
	if (fMuonFile.Length() < 3)
		fMuonFile     = "$GROPE/data/p62muon.fxddst";   
	fElectronFile     = gSystem->Getenv("ELECTRONFILE");
	if (fElectronFile.Length() < 3)
		fElectronFile = "$GROPE/data/p62electron.fxddst";
	fTauFile     = gSystem->Getenv("TAUFILE");
	if (fTauFile.Length() < 3)
		fTauFile      = "$GROPE/data/p62tt.fxddst";     
	fHadronFile     = gSystem->Getenv("HADRONFILE");
	if (fHadronFile.Length() < 3)
		fHadronFile   = "$GROPE/data/p62mh.fxddst";     
	fCosmicsFile     = gSystem->Getenv("COSMICSFILE");
	if (fCosmicsFile.Length() < 3)
		fCosmicsFile  = "$GROPE/data/cosm_muon.fxddst"; 
	fAnyEventFile     = gSystem->Getenv("ANYEVENTFILE");
	if (fAnyEventFile.Length() < 3)
		fAnyEventFile = "$GROPE/data/p62sample.fxddst"; 
	
	fODCompl["CTRK"] = 1;
	fODCompl["DTE"] = 1;
	fODCompl["ECAL"] = 1;
	fODCompl["HCAL"] = 1;
	fODCompl["MUON"] = 1;
	fODCompl["FDET"] = 1;
	fODCompl["PRES"] = 1;
	fODCompl["SIW"] = 1;
	fODCompl["TOF"] = 1;
	fODCompl["VRTX"] = 3;
	fODCompl["COLO"] = 1;
	fODCompl["TCXY"] = 1;
	fODCompl["TCSZ"] = 1;
	fODComplValue = 1;
	// type, width, color
//	Int_t defAtt[3] = {0, 1, 1};
	vector <int> defAtt(3);
	defAtt[0] = 0;
	defAtt[1] = 1;
	defAtt[2] = 1;
	TString lname;
	TString abrav;
	Int_t ltype;
	Int_t width;
	Int_t color;
	TString cmd("grep set/line ");
	cmd += gSystem->Getenv("GROPE");
	cmd+= "/data/ms.kumac";
	FILE *fp = gSystem->OpenPipe(cmd, "r");
   TString result;
	TString cval;
	TRegexp multispace(" +");
	while ( result.Gets(fp) ) {
		result = result.Strip(TString::kBoth);
		result(multispace) = ",";
		result(multispace) = ",";
		result(multispace) = ",";
		result(multispace) = ",";
		printf("line: %s\n", result.Data());
		Ssiz_t pos = 0, slen = 0;
		Bool_t ok;
		const Char_t * del = ",";
		ok = result.Tokenize(cval,pos, del);
		if ( ok ) {
			slen = cval.Length();
			result = result(slen+1, result.Length());
//			printf("cmd: %s\n", cval.Data());
		} else {
			cout << "Not enough entries: " << result << endl;
			break;
		}
		pos = 0;
		ok = result.Tokenize(lname,pos, del);
		if ( ok ) {
			slen = lname.Length();
			result = result(slen+1, result.Length());
//			printf("lname: %s\n", lname.Data());
		} else {
			cout << "Not enough entries: " << result<< endl;
			break;
		}
		pos = 0;
		ok = result.Tokenize(cval,pos, del);
		if ( ok ) {
			slen = cval.Length();
			result = result(slen+1, result.Length());
			ltype = 1;
			for (Int_t i = 0; i<kNLineTypes; i++) {
				if ( cval.Contains(gLineTypes[i]) )
					ltype = i;
			}
//			printf("ltype: %s %d\n", cval.Data(), ltype);
		} else {
			cout << "Not enough entries: " << result << endl;
			break;
		}
		pos = 0;
		ok = result.Tokenize(cval,pos, del);
		if ( ok ) {
			slen = cval.Length();
			result = result(slen+1, result.Length());
			Float_t w = cval.Atof();
			if (w < 1.)
				width = 1;
			else
				width = (Int_t)w;
//			printf("width: %s %d\n", cval.Data(), width);
		} else {
			cout << "Not enough entries: " << result << endl;
			break;
		}
		pos = 0;
		ok = result.Tokenize(cval,pos, del);
		if ( ok ) {
			slen = cval.Length();
			color = cval.Atoi();
//			printf("color: %s %d\n", cval.Data(), color);
		} else {
			cout << "Not enough entries: " << result << endl;
			break;
		}
		char ab[5];
		if ( getabrav(lname.Data(),ab) == 0 ) {
			defAtt[0] = ltype;
			defAtt[1] = width;
			defAtt[2] = color;
			fAttLine[ab] = defAtt;
			fNameLine[ab] = lname.Data();
		}
	}
	gSystem->ClosePipe(fp);
	
	attline("ODE ", 0, 1, 4);   // set ECAL colors to blue
	dst("VRTX", 3);             // draw secondary vertex lines      
	fAttLineColor = 1;
	fAttLineType  = 0;
	fAttLineWidth = 1;

	if ( gCanvas3D ) {
		fRootC3D = (TRootCanvas *)gCanvas3D->GetCanvasImp();
		if ( fRootC3D ) 
			TQObject::Connect(fRootC3D, "CloseWindow()",
							"RopeCmds", this, "HandleClose3DWindow()");
		else
			printf("ctor RopeCmds: No frame for 3D canvas!!\n");
	} else {
		printf("ctor RopeCmds: No 3D canvas!!\n");
	}
};
//______________________________________________________________________

void RopeCmds::RemoveMenu(TGMenuBar * menubar, const char * menu)
{
////////////////////////////////////////////////////////////////////////////
//                                                                        //
//                                                                        //
////////////////////////////////////////////////////////////////////////////

TGPopupMenu * pm;
	const TList * l;
   TGMenuEntry * e;
   pm = menubar->GetPopup(menu);
   if (pm) {
      l = pm->GetListOfEntries();
      if (l) {
         TIter next(l);
         while ( (e = (TGMenuEntry *)next()) ) {
            pm->DeleteEntry(e);
         }
         menubar->RemovePopup(menu);
      }
   }
}	
//______________________________________________________________________

void RopeCmds::BuildMenu()
{
	////////////////////////////////////////////////////////////////////////////
	//                                                                        //
	//                                                                        //
	////////////////////////////////////////////////////////////////////////////
	
	//   cout << "GroupOfHists::BuildMenu() " <<this << endl;
	fRootCanvas = (TRootCanvas*)gCanvasCmd->GetCanvas()->GetCanvasImp();
	TGMenuBar * menubar = fRootCanvas->GetMenuBar();

   RemoveMenu(menubar, "Tools");
   RemoveMenu(menubar, "Help");
   RemoveMenu(menubar, "File");
   RemoveMenu(menubar, "Edit");
   RemoveMenu(menubar, "Classes");
   RemoveMenu(menubar, "View");
   RemoveMenu(menubar, "Options");

	
	TGLayoutHints * layoh_left = new TGLayoutHints(kLHintsTop | kLHintsLeft);
	fMenu     = new TGPopupMenu(fRootCanvas->GetParent());
	menubar->AddPopup("Display options / Actions", fMenu, layoh_left, menubar->GetPopup("Help"));
	fMenu->AddEntry("Display options", kDisplayOptions);
	fMenu->AddEntry("Actions", kPrintData);
	fMenu->Connect("Activated(Int_t)", "RopeCmds", this,"HandleMenu(Int_t)");
	menubar->MapSubwindows();
	menubar->Layout();
}
//________________________________________________________________________

void RopeCmds::HandleMenu(Int_t id)
{
	////////////////////////////////////////////////////////////////////////////
	//                                                                        //
	//                                                                        //
	////////////////////////////////////////////////////////////////////////////
	switch (id) {
		
		case kDisplayOptions:
//			cout <<"DisplayOptions" << endl;
			DisplayOptions();
			break;
		case kPrintData:
			Actions();
			break;
	}
}
//________________________________________________________________________

void RopeCmds::DisplayOptions()
{
	////////////////////////////////////////////////////////////////////////////
	//                                                                        //
	//                                                                        //
	////////////////////////////////////////////////////////////////////////////
	static const Char_t helptext[] =
	"DisplayOptions\n\
Most items are obvious\n\
\"Color Tracks\":\n\
Set color of CD tracks according to their particle type\n\
(same color as in dedx plot) if the weight for a particle\n\
is > 0.3\n\
Changing options has effect normally only for the next event\n\
But see: \"Next\" \"Previous\"\n\
When using beamers or saving the pictures to a file lines\n\
may be too thin. A factor may be applied to increase widths\n\
for all lines or only a subset (subset not implemented yet)\n\
Colorimeter hits may be filled: Transpareny: 0 opaque; 100 transparent\n\
";
	fRow_lab = new TList();
	Int_t ind = 0;
	Int_t dummy;
	static TString setdstcmd("setdst()");
	static TString helpon_odblocks("helpdst()");
	static TString setattlinecmd("setattline()");
	static TString helpon_attline("helpattline()");
	static TString setinclwcmd("setincreaselw()");
	fRow_lab->Add(new TObjString("CheckButton_    Hits On "));
	fBidHitsOn = ind;
	fValp[ind++] = &fHitsOn;
	fRow_lab->Add(new TObjString("CheckButton+     Off "));
	fBidHitsOff = ind;
	fValp[ind++] = &fHitsOff;
	
	fRow_lab->Add(new TObjString("ComboSelect+Detector;CJ;CV;CZ;TB;PB;EB;PE;EE;HB;HE;HP;MB;ME;FD;\n\
CT;EM;HM;MM;CE;TR;MC;OD;US;SI;CS;PX"));
	fBidHitsScope = ind;
	fValp[ind++] = &fHitsScope;
	
	fRow_lab->Add(new TObjString("CheckButton_    Geom On "));
	fBidGeomOn = ind;
	fValp[ind++] = &fGeomOn;
	fRow_lab->Add(new TObjString("CheckButton+     Off "));
	fBidGeomOff = ind;
	fValp[ind++] = &fGeomOff;
	fRow_lab->Add(new TObjString("ComboSelect+Detector;CJ;CV;CZ;TB;PB;EB;PE;EE;HB;HE;HP;MB;ME;FD;\n\
CT;EM;HM;MM;CE;TR;MC;OD;US;SI;CS;PX"));
	fBidGeomScope = ind;
	fValp[ind++] = &fGeomScope;
	
	fRow_lab->Add(new TObjString("CheckButton_Color Tracks"));
	fBidColorTracks = ind;
	fValp[ind++] = &gColorTracks;
	
	fRow_lab->Add(new TObjString("CheckButton+Show Jets"));
	fBidJets = ind;
	fValp[ind++] = &fJetsOnOff;
	fRow_lab->Add(new TObjString("CheckButton+Thrust axis"));
	fBidThrust = ind;
	fValp[ind++] = &fThrustOnOff;
	
	fRow_lab->Add(new TObjString("CheckButton_Apply P Cut "));
	fBidPCut = ind;
	fValp[ind++] = &fPCutOnOff;
	fRow_lab->Add(new TObjString("Float_Value+Value[GeV]"));
	fValp[ind++] = &fPCut;
	fRow_lab->Add(new TObjString("ComboSelect+Scope;ALL;OUTER"));
	fValp[ind++] = &fPCutScope;
	
	fRow_lab->Add(new TObjString("Float_Value_Inc LWidth"));
	fValp[ind++] = &fThicken;
	fRow_lab->Add(new TObjString("ComboSelect+Scope;ALL;GEOM;TRACKSandHITS"));
	fValp[ind++] = &fThickenScope;
	fRow_lab->Add(new TObjString("CommandButt+Set"));
	fValp[ind++] = &setinclwcmd;

	fRow_lab->Add(new TObjString("CommentOnly_Draw style and transparency of Calorimeter Blocks"));
	fValp[ind++] = &dummy;
	fRow_lab->Add(new TObjString("RadioButton_Edges /Fill"));
	fValp[ind++] = &gBlockEdgesFill;
	fRow_lab->Add(new TObjString("RadioButton+Edges only"));
	fValp[ind++] = &gBlockEdges;
	fRow_lab->Add(new TObjString("RadioButton+Fill only"));
 	fValp[ind++] = &gBlockFill;
	fRow_lab->Add(new TObjString("PlainIntVal+Tra'cy"));
 	fValp[ind++] = &gTransparency;
	fRow_lab->Add(new TObjString("CommentOnly_Select OD Blocks to display and complexity level"));
	fValp[ind++] = &dummy;
	TString cmdstr("ComboSelect_ OD Block ");
	for(p = fODCompl.begin(); p != fODCompl.end(); p++) {
		cmdstr += ";";
		cmdstr += p->first;
	}
	fRow_lab->Add(new TObjString(cmdstr.Data()));
	
//	fRow_lab->Add(new TObjString("ComboSelect_OD Block;CTRK;DTE;ECAL;HCAL;MUON;FDET;PRES;SIW;TOF;VRTX;ATRK;COLO;TCXY;TCSZ"));
	fBidODBlock = ind;
	fValp[ind++] = &fODBlock;
	fRow_lab->Add(new TObjString("PlainIntVal- Value"));
	fBidODComplexity = ind;
 	fValp[ind++] = &fODComplValue;
	fRow_lab->Add(new TObjString("CommandButt-  Set  "));
	fValp[ind++] = &setdstcmd;
	fRow_lab->Add(new TObjString("CommandButt-   Help   "));
	fValp[ind++] = &helpon_odblocks;
	fRow_lab->Add(new TObjString("CommentOnly_Attributes of lines"));
	fValp[ind++] = &dummy;
	TString cmdatt("ComboSelect-");
	for(pv = fAttLine.begin(); pv != fAttLine.end(); pv++) {
		cmdatt += ";";
		cmdatt += pv->first;
	}
//	printf("cmdatt: %s\n", cmdatt.Data());
	
	fRow_lab->Add(new TObjString(cmdatt.Data()));
	fBidAttLine = ind;
	fValp[ind++] = &fAttLineName;
	fRow_lab->Add(new TObjString("CommandButt-   Help   "));
	fValp[ind++] = &helpon_attline;
	
	fRow_lab->Add(new TObjString("ColorSelect_Col"));
	fBidAttLineColor = ind;
 	fValp[ind++] = &fAttLineColor;
	fRow_lab->Add(new TObjString("PlainIntVal+Wid"));
	fBidAttLineWidth = ind;
 	fValp[ind++] = &fAttLineWidth;
	fRow_lab->Add(new TObjString("PlainIntVal+Typ"));
	fBidAttLineType = ind;
 	fValp[ind++] = &fAttLineType;
	
	fRow_lab->Add(new TObjString("CommandButt+ Set"));
	fValp[ind++] = &setattlinecmd;

	Int_t Ok = 0;
	Int_t itemwidth = 400;
	fDialog =
	new TGMrbValuesAndText("Grope Display Options", NULL, &Ok,itemwidth, fRootCanvas,
								  NULL, NULL, fRow_lab, fValp,
								  NULL, NULL, helptext, this, this->ClassName(), kIdOptions);
	Int_t xp=0, yp=0;
	UInt_t xw=0, yw=0;
	if ( fDialogCmd ) {
		xp = fDialogCmd->GetX();
		yp = fDialogCmd->GetY();
		xw = fDialogCmd->GetWidth();
		yw = fDialogCmd->GetHeight();
		printf("xp, yp, xw, yw: %d %d %d %d \n", xp, yp, xw, yw);
	}
	if (xp <= 0) {
		xp = gMainWWidth+gCmdWWidth + 16;
		yp = 0;
	} else {
		yp = yp + (Int_t)yw + 24;
	}
	// does it cross a boundary of 2 1600 screens?
	if ( gScreenWidth == 3200 ) {
		if (xp < 1600 && xp + itemwidth > 1600)
			xp = 1600;
	}
	fDialog->Move(xp,yp);
								  
}
//________________________________________________________

void RopeCmds::Actions()
{
	////////////////////////////////////////////////////////////////////////////
	//                                                                        //
	//                                                                        //
	////////////////////////////////////////////////////////////////////////////
	static const Char_t helptext[] =
	"Most items in this widget are selfexplaining\n\
DeDx displays a histogram with the specific energy\n\
for charged particles in the central jetchamber\n\
together with the theoretical curves for electrons,\n\
muons etc.\n\
Lego displays the energy/momentum deposits for the\n\
different subdetectors as function of cos(theta) vs phi\n\
\n\
\"Print\" prints OD Data blocks for the specified detector\n\
Note: Tracks, clusters are numbered 1,2 ..\n\
Block number 0 indicates print all tracks / clusters\n\
With \"Docu\" checked, values are printed one per line\n\
with description\n\
Zoom states are preserved for all views in common across\n\
events.\n\
For \"Help\" with the GL Viewer please consult its own\n\
Help menu\n\
";
	static TString printcmd("PrintODData()");
	static TString prtersecmd("printodterse()");
	static TString nextcmd("next()");
	static TString prevcmd("previous()");
	static TString evtcmd("GotoEvent()");
	static TString zoomcmd("zoom2()");
	static TString unzoomcmd("unzoom2()");
	static TString openglcmd("viewOpenGL()");
	static TString dzdispcmd("dzdisp()");
	static TString dzddivcmd("dzddiv()");
	static TString legocmd("lego()");
	static TString dedxcmd("dedx()");
	static TString quitcmd("quit()");
	
	fRow_labCmd = new TList();
	Int_t ind = 0;
	
	fRow_labCmd->Add(new TObjString("ComboSelect_Evt Type;Hadron;Electron;Muon;Tau;Cosmic;AnyEvent"));
	fBidEventType = ind;
	fValpCmd[ind++] = &fEventType;
	fRow_labCmd->Add(new TObjString("CommandButt- Get Next "));
	fValpCmd[ind++] = &nextcmd;
	fRow_labCmd->Add(new TObjString("CommandButt-Previous"));
	fValpCmd[ind++] = &prevcmd;
	fRow_labCmd->Add(new TObjString("CommandButt-  Quit  "));
	fValpCmd[ind++] = &quitcmd;
	fRow_labCmd->Add(new TObjString("PlainIntVal_Run"));
	fValpCmd[ind++] = &fRunNr;
	fRow_labCmd->Add(new TObjString("PlainIntVal+Evt"));
	fValpCmd[ind++] = &fEventNr;
	fRow_labCmd->Add(new TObjString("CommandButt+ Goto "));
	fValpCmd[ind++] = &evtcmd;
	fRow_labCmd->Add(new TObjString("CommandButt- DeDx "));
	fValpCmd[ind++] = &dedxcmd;
	fRow_labCmd->Add(new TObjString("CommandButt- Lego "));
	fValpCmd[ind++] = &legocmd;
	
	fRow_labCmd->Add(new TObjString("ComboSelect_View    ;xyView;yzView;xzView;SideView"));
	fBidViewType = ind;
	fValpCmd[ind++] = &fViewType;
	fRow_labCmd->Add(new TObjString("CommandButt-   Zoom  "));
	fValpCmd[ind++] = &zoomcmd;
	fRow_labCmd->Add(new TObjString("CommandButt-  UnZoom "));
	fValpCmd[ind++] = &unzoomcmd;
	fRow_labCmd->Add(new TObjString("CommandButt- OpenGL "));
	fValpCmd[ind++] = &openglcmd;
	
	fRow_labCmd->Add(new TObjString("ComboSelect_Detector;GNRL;CTRK;ECAL;HCAL;MUON;TOF ;VRTX;ATRK;        "));
	fValpCmd[ind++] = &fDetTypeOD;
	fRow_labCmd->Add(new TObjString("PlainIntVal-Bl Nr"));
	fValpCmd[ind++] = &fBlockNrOD;
	fRow_labCmd->Add(new TObjString("CheckButton-Docu"));
	fValpCmd[ind++] = &fWithDocuOD;
	fRow_labCmd->Add(new TObjString("CommandButt- Print "));
	fValpCmd[ind++] = &printcmd;
	fRow_labCmd->Add(new TObjString("CommandButt_Print summary"));
	fValpCmd[ind++] = &prtersecmd;
	fRow_labCmd->Add(new TObjString("CommandButt+Show Zebra banks"));
	fValpCmd[ind++] = &dzdispcmd;
	fRow_labCmd->Add(new TObjString("CommandButt+Show Zebra divs"));
	fValpCmd[ind++] = &dzddivcmd;
	Int_t Ok = 0;
	Int_t itemwidth = 400;
	fDialogCmd =
	new TGMrbValuesAndText("Grope actions", NULL, &Ok,itemwidth, fRootCanvas,
								  NULL, NULL, fRow_labCmd, fValpCmd,
								  NULL, NULL, helptext, this, this->ClassName(), kIdPrintOD);
	Int_t xp=0, yp=0;
	UInt_t xw=0, yw=0;
	if ( fDialog ) {
		xp = fDialog->GetX();
		yp = fDialog->GetY();
		xw = fDialog->GetWidth();
		yw = fDialog->GetHeight();
		printf("xp, yp, xw, yw: %d %d %d %d \n", xp, yp, xw, yw);
	}
	if (xp <= 0) {
		xp = gMainWWidth+gCmdWWidth + 16;
		yp = 0;
	} else {
		yp = yp + (Int_t)yw + 24;
	}
	// does it cross a boundary of 2 1600 screens?
	if ( gScreenWidth == 3200 ) {
		if (xp < 1600 && xp + itemwidth > 1600)
			xp = 1600;
	}
	fDialogCmd->Move(xp,yp);
}
//________________________________________________________

void RopeCmds::PrintODData()
{
	if (fWithDocuOD ) {
		printod_withdoc(fDetTypeOD.Data(), fBlockNrOD);
	} else {
		printod(fDetTypeOD.Data(), fBlockNrOD);
	}	
}
//________________________________________________________

void RopeCmds::ChangeBColor(TButton * b)
{
	if ( gB_Curr && b && b != gB_Curr ) {
		gB_Curr->SetFillColor(kGray);
		gB_Curr->Modified();
		gB_Curr->Update();
		gB_Curr = b;
		gB_Curr->SetFillColor(kGreen);
	}
}
//______________________________________________________________________

void RopeCmds::OpenDzdisp()
{
	if (gCanvasDZ == NULL)  {
		gCanvasDZ = new TCanvas("C_Grope_DzDisp", "DZDISP",10, 100, 804, 828);
		gCanvasDZ->Draw();
		gCanvasDZ->cd();
		gPad->SetLeftMargin(0);
		gPad->SetRightMargin(0);
		gPad->SetTopMargin(0);
		gPad->SetBottomMargin(0);
		TQObject::Connect((TRootCanvas *)gCanvasDZ->GetCanvasImp(), "CloseWindow()",
								"RopeCmds", this, "HandleCloseDzDispWindow()");
	} else {
		gCanvasDZ->Clear();
		gCanvasDZ->cd();
		gCanvasDZ->Pop();
		gNormTrans = 1;
	}
	if (gIniHigz == 0) {
		Int_t nw = 10000;
		iginit_(&nw);
		gIniHigz = 1;
	}
}
//________________________________________________________

void RopeCmds::DoView()
{
	if (fTotalEvents <= 0 )
		return;
	if (fViewDir == kViewXZ ) {
		xzview();
	} else if (fViewDir == kViewYZ ) {
		yzview();
	} else if (fViewDir == kViewSide ) {
		sideview();
	} else if (fViewDir == kViewXY ){
		xyview();
	}
}
//________________________________________________________

void RopeCmds::CRButtonPressed(Int_t wid, Int_t bid, TObject */*obj*/)
{
//	printf("CRButtonPressed wid %d\n", wid);
	if (wid == kIdPrintOD)  {
		if ( bid == fBidEventType ) {
			if        ( fEventType == "AnyEvent" ) {
				anyevent();
			} else if ( fEventType == "Hadron" ) {
				hadron();
			} else if ( fEventType == "Electron" ) {
				electron();
			} else if ( fEventType == "Muon" ) {
				muon();
			} else if ( fEventType == "Tau" ) {
				tau();
			} else if ( fEventType == "Cosmic" ) {
				cosmic();
			}
		} else if ( bid == fBidViewType ) {
			if        ( fViewType == "xyView" ) {
				xyview();
			} else if ( fViewType == "yzView" ) {
				yzview();
			} else if ( fViewType == "xzView" ) {
				xzview();
			} else if ( fViewType == "SideView" ) {
				sideview();
			} 
		}
	} else if (wid == kIdOptions) {
		TRegexp det("CJ");
		if ( bid == fBidHitsOn ) {
			hits( fHitsScope.Data(), 1);
			fDetHits += fHitsScope.Data();
			fDialog->SetCheckButton(fBidHitsOff,0);
	//		DoView();
		} else if ( bid == fBidHitsOff ) {
			det = fHitsScope.Data();
			hits(fHitsScope.Data(), 0);
			fDetHits(det) = "";
			fDialog->SetCheckButton(fBidHitsOn,0);
		} else if ( bid == fBidHitsScope ) {
			if( fDetHits.Contains(fHitsScope.Data())) {
				fDialog->SetCheckButton(fBidHitsOn,1);
				fDialog->SetCheckButton(fBidHitsOff,0);
			} else {
				fDialog->SetCheckButton(fBidHitsOn,0);
				fDialog->SetCheckButton(fBidHitsOff,1);
			}
		} else if (bid == fBidGeomOn ) {
			geom( fGeomScope.Data(), 1);
			fDetGeom += fGeomScope.Data();
			fDialog->SetCheckButton(fBidGeomOff,0);
			//		DoView();
		} else if ( bid == fBidGeomOff ) {
			det = fGeomScope.Data();
			geom(fGeomScope.Data(), 0);
			fDetGeom(det) = "";
			fDialog->SetCheckButton(fBidGeomOn,0);
		} else if ( bid == fBidGeomScope ) {
			if( fDetGeom.Contains(fGeomScope.Data())) {
				fDialog->SetCheckButton(fBidGeomOn,1);
				fDialog->SetCheckButton(fBidGeomOff,0);
			} else {
				fDialog->SetCheckButton(fBidGeomOn,0);
				fDialog->SetCheckButton(fBidGeomOff,1);
			}
			
		} else if (bid == fBidJets || bid == fBidThrust ) {
			recon("PX", fThrustOnOff + 2 * fJetsOnOff);
	//		DoView();
		} else if (bid == fBidPCut ) {
			if ( fPCutOnOff == 1) {
				MomentumCut(fPCut, fPCutScope.Data());
			} else {
				MomentumCut(0.);
			}
	//		DoView();
		} else if (bid == fBidODBlock ) {
			printf("fODBlock fODComplValue %s %d\n", fODBlock.Data(), fODCompl[fODBlock.Data()]);
			fODComplValue =  fODCompl[fODBlock.Data()];
			fDialog->ReloadValues();
		} else if (bid == fBidAttLine ) {
			vector<int> att = fAttLine[fAttLineName.Data()];
			string longname = fNameLine[fAttLineName.Data()];
			cout << fAttLineName.Data()<< " " << longname << " Type:"  << att[0] 
			<< " Width: "  << att[1] << " Color: " << att[2]<< endl;
			fAttLineType  = att[0];
			fAttLineWidth = att[1];
			fAttLineColor = att[2];
			fDialog->ReloadValues();
		}
	}
}
//________________________________________________________
	
void RopeCmds::viewOpenGL()
{
	if (fTotalEvents <= 0 ) {
		next();
	}
	gCanvas3D->cd();
	gGLViewer =
	(TGLViewer*)gCanvas3D->GetViewer3D("ogl");
	fFrameOGL = (TGMainFrame *)((TGLSAViewer *)gGLViewer)->GetFrame();
	fFrameOGL->Resize(gCanvas3D->GetWw(), gCanvas3D->GetWh()-124);
	fFrameOGL->Move(0, 130);
	//	mfr->Inspect();
	//	glv->SetViewport(0,0,gCanvas3D->GetWw(), gCanvas3D->GetWh()-24);
	gGLViewer->SetClearColor(1);
	(gGLViewer->RefCamera(TGLViewer::kCameraOrthoXOZ)).SetExternalCenter(kTRUE);
	(gGLViewer->RefCamera(TGLViewer::kCameraOrthoXOZ)).SetCenterVec(0,0,0);
	(gGLViewer->RefCamera(TGLViewer::kCameraOrthoZOY)).SetExternalCenter(kTRUE);
	(gGLViewer->RefCamera(TGLViewer::kCameraOrthoZOY)).SetCenterVec(0,0,0);
	(gGLViewer->RefCamera(TGLViewer::kCameraOrthoXOY)).SetExternalCenter(kTRUE);
	(gGLViewer->RefCamera(TGLViewer::kCameraOrthoXOY)).SetCenterVec(0,0,0);
	gGLViewer->SetResetCamerasOnUpdate(kFALSE);  
	//	(gGLViewer->RefCamera(TGLViewer::kCameraOrthoXOY)).SetEnableRotate(kTRUE);
	TGLCameraOverlay *co = gGLViewer->GetCameraOverlay();
	if ( co ) {
		co->SetOrthographicMode(TGLCameraOverlay::kAxis);
		co->SetShowOrthographic(kTRUE);
	}
	TGLLogicalShape::SetIgnoreSizeForCameraInterest(kTRUE);
	gGLViewer->SetCurrentCamera(TGLSAViewer::kCameraOrthoXOY);
	gGLViewer->ResetCurrentCamera();
	//	gGLViewer->UpdateScene();
	TQObject::Connect(fFrameOGL, "CloseWindow()",
							"RopeCmds", this, "HandleCloseOGLWindow()");
	TQObject::Connect(gGLViewer, "MouseOver(TObject*,UInt_t)",
							"RopeCmds", this, "HandleMouseOver(TObject*,UInt_t)");
	TQObject::Connect(gGLViewer, "Clicked(TObject*, UInt_t, UInt_t",
							"RopeCmds", this, "HandleClicked(TObject*, UInt_t, UInt_t)");
}
//__________________________________________________________

void RopeCmds::HandleMouseOver(TObject* obj,UInt_t val)
{
	if (obj) 
		printf("MouseOver: %s\n", obj->GetTitle());
}
//__________________________________________________________

void RopeCmds::HandleClicked(TObject* obj, UInt_t button, UInt_t state)
{
	if (obj && button == 1) {
//		printf("HandleClicked: %s %i %i\n", obj->GetName(), button, state);
		TString na(obj->GetName());
		TString name = na(4,4);
		na = na(9,name.Length());
		Int_t nr = na.Atoi();
		printod(name.Data(), nr);
	}
}
//__________________________________________________________

void RopeCmds::HandleCloseCmdWindow()
{
	printf("HandleCloseCmdWindow: Terminate Grope\n");
	gSystem->Exit(0);
}
//__________________________________________________________

void RopeCmds::HandleCloseDzDispWindow()
{
	if ( gCanvasDZ ) {
		printf("HandleCloseDzDispWindow: \n");
		gCanvasDZ->cd();
		// dirty trick to avoid updating during delete
		gCanvasDZ->SetSupportGL(kTRUE);
		// force return from TPad::WaitPrimitive
		gROOT->SetSelectedPad(0);
//		gSystem->Sleep(100);
		gCanvasDZ = NULL;
	}
}
//__________________________________________________________

void RopeCmds::HandleClose3DWindow()
{
//	printf("HandleClose3DWindow: Terminate Grope\n");
	gCanvas3D = NULL;
	gSystem->Exit(0);
}
//__________________________________________________________

void RopeCmds::HandleCloseOGLWindow()
{
//	printf("HandleCloseOGLWindow:\n");
	gGLViewer = NULL;
}
//________________________________________________________

void RopeCmds::SetColorTracks(Int_t docolor) 
{ 
	gColorTracks = docolor; 
}; 
//________________________________________________________

void RopeCmds::ClearEventNumbers() 
{ 
	while ( !fEventNumbers.empty() ) {
		Int_t ievt = fEventNumbers.top();
		Int_t irun = fRunNumbers.top();
		
		printf ("Remove run, event: %d %d\n", irun, ievt);
		fEventNumbers.pop();
		fRunNumbers.pop();
	}
}
//________________________________________________________
	
void RopeCmds::StoreEventNumber() 
{ 
	gODpbSwitch = 1;
	Int_t n = 0;
	odpblk_("GNRL", &n);
	if (gNumberIntOD < 10) {
		printf("Not enough GNRL words %d\n", gNumberRealOD);
		return;
	}
	Int_t ievt = gIntOD[5];
	Int_t irun = gIntOD[3];
	fEventNumbers.push(ievt);
	fRunNumbers.push(irun);
	fTotalEvents++;
	printf ("Run, event: %d %d\n", irun, ievt);
}; 
//__________________________________________________________

void RopeCmds::GotoEvent()
{
// Goto to event fEventNr, fRunNr
// Called from command button
	printf ("Goto run, event: %d %d\n", fRunNr, fEventNr);
	rewind();
	getevent(fEventNr, fRunNr);
	rsevct();    // reset event counters in rope
	if ( GetNwGNRL() < 10 ) {
		printf("Event not found\n");
		rewind();
	} else {
		DoView();
	}
}
//__________________________________________________________

void RopeCmds::previous()
{
// Get previous event

	if ( fEventNumbers.empty() ) {
		printf("Already at start\n");
		return;
	}
	fEventNumbers.pop();
	fRunNumbers.pop();
	if ( fEventNumbers.empty() ) {
		printf("Already at start\n");
		return;
	}
	Int_t ievt = fEventNumbers.top();
	Int_t irun = fRunNumbers.top();
	
	printf ("previous run, event: %d %d\n", irun, ievt);
	rewind();
	getevent(ievt, irun);
	DoView();
}
//__________________________________________________________

Int_t RopeCmds::GetNwGNRL()
{
// Get number of integer words in General block
// Indicates if event has been read successfully

//	printf("Print OD: %s %i\n", name, nr);
	gBlockName = "GNRL";
	gBlockNumber = 0;
	gBlockSeq = 0;
	gODpbSwitch = 1;
	odpblk_("GNRL", &gBlockNumber);
	printf("GetNwGNRL:  %i\n", gNumberIntOD);
	gODpbSwitch = 0;
	return gNumberIntOD;
}
//__________________________________________________________

void RopeCmds::printod(const char * name, Int_t nr)
{
	// Print data for one OD block 
	// GNRL;CTRK;ECAL;HCAL;MUON;TOF ;VRTX;ATRK
	// 
	printf("Print OD: %s %i\n", name, nr);
	gBlockName = name;
	gBlockNumber = nr;
	gBlockSeq = nr;
	gODpbSwitch = 0;
	odpblk_(name, &nr);
}
//________________________________________________________

void RopeCmds::printod_withdoc(const char * name, Int_t nr)
{
// Print data for one OD block with description, one line/value 
// GNRL;CTRK;ECAL;HCAL;MUON;TOF ;VRTX;ATRK
// 
	gODpbSwitch = 2;
	gBlockName = name;
	gBlockNumber = nr;
	gBlockSeq = nr;
	odpblk_(name, &nr);
	gODpbSwitch = 0;
	
}
//__________________________________________________________

void RopeCmds::printodterse()
{
// Print a summary of event, one line/track - cluster
	cmd = "ALL ";
	n_args = 0;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	odinpr_();
}
//__________________________________________________________

void RopeCmds::rotrig()
{
// Get next event

	cmd = "TRIG";
	n_args = 0;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	rogoku_();
}
//__________________________________________________________

void RopeCmds::rewind()
{
// Rewind current input file
	cmd = "REWIND";
	n_args = 0;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	rogoku_();
	rsevct();    // reset event counters in rope
}
//__________________________________________________________

void RopeCmds::getevent(Int_t ev, Int_t run)
{
// Get event by number and run

	cmd = "GOTO";
	n_args = 3;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	argi[0] = ev;
	argi[1] = run;
	*(argc[0]) = "CURRENT";
	printf("getevent: %d %d\n", ev, run);
	rogoku_();
	if ( gCanvas3D ) {
		gCanvas3D->Clear();
		gCanvas3D->cd();
		view3d_();
	} else {
		//		popwk_();
	}
	redraw();
	rsevct();    // reset event counters in rope
//	StoreEventNumber();
}
//__________________________________________________________

void RopeCmds::skip(Int_t nev, Int_t nproc)
{
	cmd = "SKIP";
	n_args = 3;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	argi[0] = nev;
	argi[1] = nproc;
	printf("skip: %d %d\n", nev,nproc);
	rogoku_();
}
//__________________________________________________________
	
void RopeCmds::trig()
{
// Get and display next event, update lego and dedx plots if 
// they are displayed
	
	if ( gCanvasDZ ) {
		gCanvasDZ->Close();
		gCanvasDZ = NULL;
	}
	grgevt_();
	if (gROOT->GetListOfCanvases()->FindObject("C_Grope_Lego") ) {
		RopeCmds::lego();
	}
	if (gROOT->GetListOfCanvases()->FindObject("C_Grope_dedx") ) {
		RopeCmds::dedx();
	}
	DoView();
	StoreEventNumber();
};
//________________________________________________________

void RopeCmds::unzoom2()
{
// UnZoom by factor 2

	gCanvas3D->cd();
	fZoom /= 2.;
	gView3D->ZoomView((TVirtualPad*)gCanvas3D, 0.5);
	if (gScaleText) {
		gScale *= 2;
		TString scale;
		scale+= gScale;
		scale += " cm.";
		gScaleText->SetText(gScaleText->GetX(), gScaleText->GetY(),scale);
		gCanvas3D->Modified();
		gCanvas3D->Update();
	}
	DoView();
}
//________________________________________________________

void RopeCmds::zoom2()
{
	
// Zoom by factor 2

	gCanvas3D->cd();
	fZoom *= 2;
	gView3D->ZoomView((TVirtualPad*)gCanvas3D, 2);
	if (gScaleText) {
		gScale /= 2;
		TString scale;
		scale+= gScale;
		scale += " cm.";
		gScaleText->SetText(gScaleText->GetX(), gScaleText->GetY(), scale);
		gCanvas3D->Modified();
		gCanvas3D->Update();
	}
	DoView();
};
//________________________________________________________
	
void RopeCmds::xzview()
{
// view from side

	gCanvas3D->cd();
//	gView3D->Front();
	ChangeBColor (gB_xzView);
	fViewDir = kViewXZ;
	gView3D->RotateView(270., 90., gCanvas3D);
	if ( gGLViewer ) {	
		gGLViewer->SetCurrentCamera(TGLViewer::kCameraOrthoXOZ);
		gGLViewer->SetOrthoCamera(TGLViewer::kCameraOrthoXOZ, fZoom,0.,fCenter, 0, 0);
		gGLViewer->DoDraw();
	}
};
//________________________________________________________

void RopeCmds::yzview()
{
	// view from top
	
	gCanvas3D->cd();
	gView3D->RotateView(180., 90., gCanvas3D);
	ChangeBColor(gB_yzView);
	fViewDir = kViewYZ;	
	if ( gGLViewer ) {
		gGLViewer->SetCurrentCamera(TGLViewer::kCameraOrthoZOY);
		TGLCamera& o = gGLViewer->CurrentCamera();
		TGLOrthoCamera *oc = (TGLOrthoCamera*)&o;
		oc->SetEnableRotate(kTRUE);
//		printf("yzview %f\n", fYZCameraAngle);
//		if (fYZCameraAngle != 0 ) {
//			gGLViewer->SetOrthoCamera(TGLViewer::kCameraOrthoZOY, fZoom,0.,fCenter, -fYZCameraAngle, 0);
//			fYZCameraAngle = 0;
//		}
		gGLViewer->ResetCurrentCamera();
		gGLViewer->SetOrthoCamera(TGLViewer::kCameraOrthoZOY, fZoom,0.,fCenter, 0, 0);
		gGLViewer->DoDraw();
	}
};
//________________________________________________________

void RopeCmds::xyview()
{
// View along beam line

	gCanvas3D->cd();
//	gView3D->Top();
	gView3D->RotateView(270., 0., gCanvas3D);
	ChangeBColor(gB_xyView);
	fViewDir = kViewXY;
	if ( gGLViewer ) {
		gGLViewer->SetCurrentCamera(TGLViewer::kCameraOrthoXOY);
		gGLViewer->SetOrthoCamera(TGLViewer::kCameraOrthoXOY, fZoom,0.,fCenter,0,0);
		gGLViewer->DoDraw();
//		gGLViewer->UpdateScene();
	}
}
//________________________________________________________

void RopeCmds::sideview(Int_t ISIDE)
{
// Viewing direction perpedicular to the plane beamline - thrustaxis

	gCanvas3D->cd();
	gODpbSwitch = 1;
	Int_t elmid = 0;
	odpblk_("GNRL", &elmid);
	gODpbSwitch = 0;
	if (gNumberRealOD < 30) {
		printf("Not enough GNRL words %d\n", gNumberRealOD);
		return;
	}
	ChangeBColor(gB_SideView);
	fViewDir = kViewSide;
	gView3D->Side();
	Float_t phi_thrust = 180 * gRealOD[30] / TMath::Pi() + 90.;
	if (phi_thrust > 360)
		phi_thrust -= 360;
//	printf("phi_thrust %f\n", phi_thrust);
	gView3D->RotateView(phi_thrust, 90, gCanvas3D);
	if ( gGLViewer ) {
		gGLViewer->SetCurrentCamera(TGLViewer::kCameraOrthoZOY);
		TGLCamera& o = gGLViewer->CurrentCamera();
		TGLOrthoCamera *oc = (TGLOrthoCamera*)&o;
		oc->SetEnableRotate(kTRUE);
		Double_t phi = gRealOD[30];
		if ( phi <= TMath::Pi() ) {
			phi -= 0.5 * TMath::Pi();
		} else {
			phi -= 1.5 * TMath::Pi();
		}	
		if (fYZCameraAngle != 0 ) {
//			printf("fYZCameraAngle: %f\n", fYZCameraAngle);
//			gGLViewer->SetOrthoCamera(fViewDir, fZoom,0.,fCenter, -fYZCameraAngle, 0);
		}
//		printf("sideview phi:  raw  %f, corr %f\n", gRealOD[30], phi);
		gGLViewer->ResetCurrentCamera();
		fYZCameraAngle = phi;
		gGLViewer->SetOrthoCamera(TGLViewer::kCameraOrthoZOY, fZoom,0,fCenter, phi, 0);
		gGLViewer->DoDraw();
	}
}
//________________________________________________________

void RopeCmds::colorctonoff()
{
// Toggle coloring of CT tracks according to the particle ID
// colors are as in dedx plot
	if ( gColorTracks ) {
		gColorTracks = 0;
		gB_ColorCT->SetFillColor(kGray);
	} else {
		gColorTracks = 1;
		gB_ColorCT->SetFillColor(kGreen);
	}
	DoView();
}
//________________________________________________________

void RopeCmds::thrustonoff()
{
// Toggle display of thrust and missing energy vectors

	if ( fThrustOnOff ) {
		fThrustOnOff = 0;
		gB_ThrustOnOff->SetFillColor(kGray);
	} else {
		fThrustOnOff = 1;
		gB_ThrustOnOff->SetFillColor(kGreen);
	}
	recon("PX", fThrustOnOff + 2 * fJetsOnOff);
	DoView();
};
//________________________________________________________
void RopeCmds::jetsonoff()
{
// Toggle display of jet cones

	if ( fJetsOnOff ) {
		fJetsOnOff = 0;
		gB_JetsOnOff->SetFillColor(kGray);
	} else {
		fJetsOnOff = 1;
		gB_JetsOnOff->SetFillColor(kGreen);
	}
	recon("PX", fThrustOnOff + 2 * fJetsOnOff);
	DoView();
};
//________________________________________________________

void RopeCmds::geom(const char * idet, Int_t iflag, Int_t isect)
{
// Define display of the geometry layout of OPAL detectors
// IDET  'Detector component' C  D='  ' R='CV,CJ,CZ,TB,PB,EB,PE,EE,HB,HE,HP,MB,ME,FD,CT,EM,HM,MM,CE,TR,MC,OD,US,SI,CS,PX'
// IFLAG 'Complexity level'   I  R=0:
// IFLAG 'Complexity level'   I  R=0:

	cmd = "GEOM";
	n_args = 3;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	*(argc[0]) = idet;
	argi[0] = iflag;
	argi[1] = isect;
	groset_();
}
//__________________________________________________________

void RopeCmds::hits(const char * idet, Int_t iflag, Int_t isect)
{
// Define which raw data should be shown: hits etc.
// IDET  'Detector component' C  D='  ' R='CV,CJ,CZ,TB,PB,EB,PE,EE,HB,HE,HP,MB,ME,FD,CT,EM,HM,MM,CE,TR,MC,OD,US,SI,CS,PX'
// IFLAG 'Complexity level'   I  R=0:
// ISECT 'Sector number (if applicable)' I R=0: D=0

	cmd = "HITS";
	n_args = 3;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	*(argc[0]) = idet;
	argi[0] = iflag;
	argi[1] = isect;
	printf("hits: %s %d %d\n", idet, iflag, isect);
	groset_();
	DoView();
}
//________________________________________________________

void RopeCmds::recon(const char * idet, Int_t iflag, Int_t isect)
{
// Define display of reconstructed ROPE objects: tracks, clusters etc.
// IDET  'Detector component' C  D='  ' _
// R='CV,CJ,CZ,TB,PB,EB,PE,EE,HB,HE,HP,MB,ME,FD,_
// CT,EM,HM,MM,CE,TR,MC,OD,US,SI,CS,PX'
//IFLAG 'Complexity level'   I  R=0:
// ISECT 'Sector number (if applicable)' I R=0: D=0
	
	cmd = "RECO";
	n_args = 3;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	*(argc[0]) = idet;
	argi[0] = iflag;
	argi[1] = isect;
	groset_();
	redraw();
}
//______________________________________________________________________

void RopeCmds::dzdisp(const char * ctop, Int_t numid, const char * mtyp)
{
	
// CALL ZEBRA bank display from the DZDOC package
// CTOP 'Name of top bank' C D='REV '
// NUMID    'Numerical bank Id' I D=0
// MTYP     'Metafile type (POSTscript,TEK,GKS)?' C D='POST' R='POST,TEK,GKS'

	OpenDzdisp();
   cmd = "DZDISP";
   n_args = 3;
   argi_count = 0;
   argf_count = 0;
   argc_count = 0;
   *(argc[0]) = ctop;
   argi[0] = numid;
   *(argc[1]) = mtyp;
   grdzgo_();
}
//______________________________________________________________________

void RopeCmds::dzddiv(const char * ctop, const char * mtyp)
{
// CALL division display from the DZDOC package
	OpenDzdisp();
   cmd = "DZDDIV";
   n_args = 2;
   argi_count = 0;
   argf_count = 0;
   argc_count = 0;
// CTOP 'Name of div' C D='IRODIV'
   *(argc[0]) = ctop;
// MTYP     'Metafile type (POSTscript,TEK,GKS)?' C D='POST' R='POST,TEK,GKS'
   *(argc[1]) = mtyp;
   grdzgo_();
}
//________________________________________________________

void RopeCmds::dedx(Int_t itrk, Float_t dedxlo, Float_t dedxhi)
{
// Command DEDX
//	ITRK  'Track number (0=all good; -1=all)' I D=0 R=-1:
//	DEDXLO 'Lower bound on dE/dx plot' R D=5.
//	DEDXHI 'Upper bound on dE/dx plot' R D=55.

	cmd = "DEDX";
	n_args = 3;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	argi[0]    = itrk;
	argf[0] = dedxlo;
	argf[1] = dedxhi;
	grdedx_();
};
//________________________________________________________

void RopeCmds::lego(const char * name, Float_t theta, Float_t phi, const char * iopr)
{
//	Command LEGO
// NAME 'Name (ECAL, HCAL, CTRK, ALL, CTEC, CTHC, ECHC)' C D='ALL ' R='ECAL,HCAL,CTRK,ALL,CTEC,CTHC,ECHC'
// THETA 'Theta to view from' R D=45. R=0.:90.
// PHI   'Phi to view from' R D=45. R=0.:90.
// IOPR  'Hplot option (e.g LOGZ)' C D='LINZ'
	
	cmd = "LEGO";
	n_args = 4;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	*(argc[0]) = name;
	argf[0] = theta;
	argf[1] = phi;
	*(argc[1]) = iopr;
	grlego_();
}

void RopeCmds::siroll(int iladd, int iflag, float zmin, float zmax, float phmin, float phmax)
{
	cmd = "SIROLL";
	n_args = 6;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	// ILADD  'Ladder number (0 => all; -ve => track number) ' I D=0
	argi[0] = iladd;
	// IFLAG  'Set non-zero to colour code matches ' I D=0 R=-1:4
	argi[1] = iflag;
	// ZMIN   'Lower limit on z'                 R D=-9. R=-15.5:15.5
	argf[0] = zmin;
	// ZMAX   'Upper limit on z'                 R D=+9. R=-15.5:15.5
	argf[1] = zmax;
	// PHMIN  'Lower limit on phi'               R D=0.  R=-3.2:6.4
	argf[2] = phmin;
	// PHMAX  'Upper limit on phi'               R D=6.2832 R=-3.2:6.4
	argf[3] = phmax;
	siroll_();
}
//________________________________________________________

void RopeCmds::file(const char * name, const char * type, const char * dir, 
						  Int_t recl,  const char * zopt, const char * ropt)
{
// /home/f/fpopal/grope/data/p62muon.fxddst data in 0 LXI
	TString fname(name);
	if (fname.BeginsWith("$")) {
		Int_t pp = fname.Index("/");
		if (pp > 0) {
			TString var = fname(1,pp-1);
			printf("var : %s\n", var.Data());
			var = gSystem->Getenv(var);
			fname = fname(pp, fname.Length());
			fname.Prepend(var);
			printf("expand filename to: %s\n", fname.Data());
		}
	}
	cmd = "FILE";
	n_args = 6;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	*(argc[0]) = fname.Data();
	*(argc[1]) = type;
	*(argc[2]) = dir;
	argi[0]    = recl;
	*(argc[3]) = zopt;
	*(argc[4]) = ropt;
	rofile_();
}

/*void RopeCmds::SetPCut(Float_t p) 
{ 
	fPCut = p;
	if (p == 0.) {
		fPCutOnOff = 1;
	} else {
		fPCutOnOff = 0;
	}
	pcutonoff();
};*/
//________________________________________________________

void RopeCmds::pcutonoff()
{
	if ( fPCutOnOff ) {
		fPCutOnOff = 0;
		gB_ApplyPCut->SetFillColor(kGray);
		MomentumCut(0.);
	} else {
		fPCutOnOff = 1;
		gB_ApplyPCut->SetFillColor(kGreen);
		MomentumCut(fPCut, fPCutScope);
	}
	gB_ApplyPCut->Modified();
	gB_ApplyPCut->Update();
}
//________________________________________________________

void RopeCmds::MomentumCut(Float_t p, const char * scope)
// >Command MOMENTUMCUT
// >Parameters
// PMIN 'Minimum momentum/energy? (GeV)' R D=0. R=0.:
// +
// SCOPE    'ALL, or just OUTER detectors' C D='ALL' R='ALL,OUTER'
// >Guidance 
// Defines the minimum momentum/energy for displaying the OD/TREE information.
// >Action GROSET
{
	cmd = "MOMENTUMCUT";
	n_args = 2;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	argf[0] = p;
	*(argc[0]) = scope;
	groset_();
	if (p > 0)
		fPCutOnOff = 1;
	else
		fPCutOnOff = 0;
	fPCut = p;
	fPCutScope = scope;
	DoView();
	printf("momentumcut %f\n", p);
}
//________________________________________________________

void RopeCmds::fillbox(Int_t igrfil , const char * igrfed, Int_t igrisi)
// >Command FILLBOX
// >Parameters
// IGRFIL  'Fill area style? (0=hollow, 1=solid, 2=pattern, 3=hatch )' I D=0 _
// R=0,1,2,3
// +
// IGRFED  'Line around edges?' C R='ON,OFF'  D='ON'
// IGRISI  'Hatch style index? (1000*step(pxl)+angle(deg))' I D=3045 R=1000:
// >Guidance
// Determines whether boxes have sides drawn as fill areas.
// >Action GROSET
{
	cmd = "FILLBOX";
	n_args = 3;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	argi[0] = igrfil;
	argi[1] = igrisi;
	*(argc[0]) = igrfed;
	groset_();
}
//________________________________________________________

void RopeCmds::setattline()
{
	attline(fAttLineName,fAttLineType, fAttLineWidth, fAttLineColor);
}	
//________________________________________________________

void RopeCmds::attline(const char * btype, Int_t type, Int_t width, Int_t color)
{
	cmd = "LINESTYLE";
	const char * lt[5] = {"SOLID   ", "DASH    ", "DOT     ", "DASHDOT "};
	Int_t ntype = 0;
	if (type >=0 && type <= 4)
		ntype = type;
	n_args = 4;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	argf[0] = width;
	argi[0] = color;
	*(argc[0]) = btype;
	*(argc[1]) = lt[ntype];
	grsere_();
	// update list
	vector<int> att = fAttLine[btype];
	att[0] = type ;
	att[1] = width;
	att[2] = color;
	string longname = fNameLine[btype];
	cout << " Set attr for: " << longname << " Type:"  << att[0] 
	     << " Width: "  << att[1] << " Color: " << att[2]<< endl;
	fAttLine[btype] = att;
}
//________________________________________________________

void RopeCmds::helpattline()
{
static const Char_t helptext_attline[] =
">Command LINESTYLE\n\
>Parameters\n\
OPTION \'Line-Style Name\' C D=\'    \'\n\
-AX   AxisLine\n\
-BU   ButtonsLine\n\
+CDR  CDRecLine\n\
+CJG  CJGeoLine\n\
+CJT  CJTrackLine\n\
-CVG  CVGeoLine\n\
-CVGA CVGeoAnodeLine\n\
+CVR  CVRecLine\n\
-CVRU CVRecUnmatcLine\n\
-CZG  CZGeoLine\n\
-CZR  CZRecLine\n\
-EBG  EBGeoLine\n\
-EBH  EBHitLine\n\
-EEG  EEGeoLine\n\
-EEH  EEHitLine\n\
-EMR  EMRecLine\n\
-EXR  EXRecLine\n\
-MERG MERgeoline\n\
-MEHL MEHitline\n\
-TBH  TOFHitLine\n\
-ZB   ZBoxLine\n\
-GRTC GRTreeCHadrLine\n\
-GRTE GRTreeElectLine\n\
-GRTM GRTreeMuonLine\n\
-GRTH GRTreeNHadrLine\n\
-GRTN GRTreeNuLine\n\
-GRTP GRTreePhotoLine\n\
-GRTT GRTreeTauLine\n\
-HEAD HeaderLine\n\
-HBH  HBHitLine\n\
-HBS  HBStripsLine\n\
-2BOX 2DBoxline\n\
-HEH  HEHitLine\n\
-HES  HEStripsLine\n\
-HPH  HPHitLine\n\
-HPS  HPStripsLine\n\
-MBG  MBGeoLine\n\
-MEG  MEGeoLine\n\
-MEH  MEHitLine\n\
-MES  MEStripsLine\n\
-MUH  MUHadStrLine\n\
-MUR  MURecMEMBLine\n\
-MUMG MURecMMMergLine\n\
-MUNM MURecMMNoMgLine\n\
+ODCB ODCTBadChiLine\n\
+ODCG ODCTGoodChiLine\n\
+ODCM ODCTMediChiLine\n\
+ODE  ODECalLine\n\
+ODF  ODFDetLine\n\
+ODHB ODHCalBadLine\n\
+ODHG ODHCalGoodLine\n\
+ODHM ODHCalMuonLine\n\
+ODM  ODMuonLine\n\
+ODPC ODPresCoLine\n\
+ODPB ODPresBWLine\n\
+ODSB ODSecVrtxBWLine\n\
+ODSC ODSecVrtxCoLine\n\
-STD  StandardLine\n\
-ECL1 EtoCLine1\n\
-ECL2 EtoCLine2\n\
-ECL3 EtoCLine3\n\
-ECL4 EtoCLine4\n\
-ECL5 EtoCLine5\n\
-ECL6 EtoCLine6\n\
-ECL7 EtoCLine7\n\
-HPR  HPRecLine\n\
-PBG  PBGeoLine\n\
-PBH  PBHitLine\n\
-PBR  PBRecLine\n\
-TBRL TOFROutLine\n\
-THR  ThrustLine\n\
-PVIS PmisLine\n\
-FDR  FDetRecLine\n\
-MELG MELgeoline\n\
-TBDL TOFROutDummLine\n\
-TBDL HBStripEdgeLine\n\
-TBDL HBStripDeadLine\n\
-SIB  SIBarrelLine\n\
-SIL  SILadderLine\n\
-SID  SIDetectorLine\n\
-SIH  SIHitsLine\n\
-SIR  SIReconLine\n\
-ODSW ODSiWLine\n\
-FBOX FillBoxLine\n\
+JETC JetsLineCone\n\
-JETS JetsLineJade\n\
-IDEL IDElectronLine\n\
-IDMU IDMuonLine\n\
-IDLA IDLambdaLine\n\
-IDKS IDKshortLine\n\
-IDGC IDGammaConvLine\n\
-COMB CombineLine\n\
TYPEST \'Line style\' C R='SOLID,DASH,DOT,DASHDOT\' D=\'SOLID\'\n\
WIDTH \'Line width\' R D=0.5 R=0.0:10.0\n\
COLI \'Line color index\' I D=1 R=0:15\n\
>Guidance\n\
Defines one (or all) line style.\n\
>Action GRSERE\n\
";
	TString temp(helptext_attline);
	Int_t nl = temp.CountChar('\n');
	nl *= 15;
	TRootHelpDialog * hd = new TRootHelpDialog(fRootCanvas, "Line Attributes", 550, nl+25);
	hd->SetText(helptext_attline);
	hd->Popup();
}
//________________________________________________________

void RopeCmds::setdst()
{
	dst(fODBlock, fODComplValue);
}	
//________________________________________________________

void RopeCmds::dst(const char * btype, Int_t clevel)
{
	cmd = "DST ";
	n_args = 2;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	argi[0] = clevel;
	*(argc[0]) = btype;
	groset_();
	fODCompl[btype] = clevel;
}
//________________________________________________________

void RopeCmds::helpdst()
{
static const Char_t helptext_dst[] =
">Command DST\n\
>Parameters\n\
IBLK  'OD Block Type; or COLOur or TCXY or TCSZ ' C D='    ' _\n\
R='CTRK;DTE;ECAL;HCAL;MUON;FDET;PRES;SIW;TOF;VRTX;ATRK;COLO;TCXY;TCSZ'\n\
IFLAG 'Complexity level'   I  R=0:4\n\
>Guidance\n\
Select which types of OD blocks are drawn within the OD display\n\
and with which level of complexity.   The following are available:\n\
 CTRK  IFLAG= 1 Draw tracks; extrapolated to their parent vertex\n\
                if known.\n\
       IFLAG= 2 Draw track numbers by the ends of tracks.\n\
       IFLAG= 3 Draw track extrapolations to ECAL HCAL and MUON; as\n\
                arrows.  Length of arrow = 20cm*SQRT(p).  Cross bars\n\
                on tail of arrow show errors in extrapolated theta/phi.\n\
                Also for those tracks drawn back to d0 draws cross\n\
                bars to show errors in d0 and z0.\n\
       IFLAG= 4 equivalent to 2+ 3combined.\n\
 DTE   IFLAG= 1 Draw TE panels hit.\n\
 ECAL  IFLAG= 1 Draw ECAL clusters as boxes.\n\
       IFLAG= 2 Label ECAL clusters with cluster numbers.\n\
       IFLAG= 3 Draw a cross on the inner face of the box to represent t\n\
                errors on theta/phi.\n\
       IFLAG= 4 equivalent to 2+ 3combined.\n\
 FDET  IFLAG= 1 Draw FDET clusters as boxes.\n\
       IFLAG= 2 Label FDET clusters with cluster energies.\n\
       IFLAG= 3 Draw a cross on the inner face of the box to represent t\n\
                errors on theta/phi.\n\
       IFLAG= 4 equivalent to 2+ 3combined.\n\
 HCAL  IFLAG= 1 Draw HCAL clusters as boxes\n\
       IFLAG= 2 Label HCAL clusters with cluster numbers.\n\
       IFLAG= 3 Draw a cross on the inner face of the box to represent t\n\
                errors on theta/phi.\n\
       IFLAG= 4 equivalent to 2+ 3combined\n\
 MUON  IFLAG= 1 Draw Muon tracks as arrows.\n\
       IFLAG= 2 Label with muon track number.\n\
       IFLAG= 3 Draw a cross on the tail of the arrow to represent the\n\
                errors on (x;y;z).\n\
       IFLAG= 4 equivalent to 2+ 3combined.\n\
 PRES  IFLAG= 1 Draw PRES clusters as rectangles.\n\
       IFLAG= 3 Draw a cross in the rectangle to represent the\n\
                errors on theta/phi.\n\
 SIW   IFLAG= 1 Draw SIW  clusters as boxes.\n\
       IFLAG= 2 Label SIW  clusters with cluster energies.\n\
 TOF   IFLAG= 1 Draw TOF bars hit.  If both ends gave good signals the\n\
                box will be solid; otherwise hollow.\n\
       IFLAG= 2 Draw the time-of-flight (ns) by each bar.\n\
 VRTX  IFLAG= 1 Draw the main vertex; as an asterisk; surrounded by the\n\
                error ellipsoid.\n\
       IFLAG= 2 Also draw \"ATRK\"'s connecting main vertex to secondaries.\n\
This command can also be used to set colour coding of energies on or off\n\
(SET/DST COLO 1 or 0) or to modify the track parameters using the main\n\
vertex and its errors to constrain CTRKs to pass closer to the main vertex\n\
on display (SET/DST TCXY 1 or SET/DST TCSZ 1).\n\
>Action GROSET\n\
";
	TString temp(helptext_dst);
	Int_t nl = temp.CountChar('\n');
	nl *= 15;
	TRootHelpDialog * hd = new TRootHelpDialog(fRootCanvas, "OD Blocks Complexity", 550, nl+25);
	hd->SetText(helptext_dst);
	hd->Popup();
};
//________________________________________________________

void RopeCmds::save_attributes(const char * filename)

// >Command SAVE_ATTRIBUTES
// >Parameters
// FILENAME 'File name' C D='mysave.kumac'
// >Guidance
// Saves current non-standard attribute settings to a Kumac for future use.
// >Action GRSAVE
{
	cmd = "SAVE_ATTRIBUTES";
	n_args = 1;
	argi_count = 0;
	argf_count = 0;
	argc_count = 0;
	*(argc[0]) = filename;
	grsave_();
}
//________________________________________________________

Int_t RopeCmds::getabrav(const char * name, char abrav[5])
{
	char ab[5];
	char na[16];
	int len = strlen(name);
	if (len > 15) len = 15;
	for (Int_t i=0; i<15; i++){
		if (i <len)
			na[i] = name[i];
		else
			na[i]= ' ';
	}
	na[15]= 0;
	int ierr;
//	printf("name: %s %s \n", name, na);
	grplna_f_(na, &len,ab,&ierr);
	
//	printf("na: %s abrav: %s %d\n", na, ab, ierr);
	for (Int_t i=0; i<4; i++)
		abrav[i] = ab[i];
	abrav[4]= 0;

	return ierr;
}	
//________________________________________________________

void RopeCmds::setincreaselw()
{
	gThickenGeom = gThickenOD = 1;
	if (fThickenScope.Contains("GEOM") || fThickenScope.Contains("ALL"))
		gThickenGeom = fThicken;
	else if (fThickenScope.Contains("GEOM") || fThickenScope.Contains("TRACKS"))
		gThickenOD = fThicken;
}
