#ifndef KUIP2ROOT
#define KUIP2ROOT
#include "TCanvas.h"
#include "TRootCanvas.h"
#include "TButton.h"
#include "TGFrame.h"
#include "TObject.h"
#include "TString.h"
#include "TSystem.h"
#include "TView3D.h"
#include "TGLViewer.h"
#include "TMath.h"
#include <TGMenu.h>
#include "TGMrbValuesAndText.h"
#include "stdio.h"
#include <map>
#include <stack>
#include <vector>

//______________________________________________________

extern "C" void rsevct_();
extern "C" void grgevt_();
extern "C" void grdraw_();
extern "C" void grdedx_();
extern "C" void grlego_();
extern "C" void siroll_();
extern "C" void rofile_();
extern "C" void rogoku_();
extern "C" void groset_();
extern "C" void grsere_();
extern "C" void grdzgo_();
extern "C" void grsave_();
extern "C" void grplna_f_(const char * name, int * len, char * abr, int * ierr);
extern "C" void grupd_();
extern "C" void odinpr_();
extern "C" { void odpblk_( const char * c,  int * n );}
extern "C" void view3d_();
extern "C" void iginit_(Int_t * nw);

extern TCanvas * gCanvas3D;
extern TCanvas * gCanvasCmd;
extern TView3D * gView3D;

extern TGLViewer * gGLViewer;

using namespace std;
//______________________________________________________

class RopeCmds : public TObject
{
private:
	typedef map<std::string, Int_t> odcompl;
	map<std::string, Int_t>::iterator p;
	odcompl fODCompl;
	typedef map<std::string, vector<int> > attlinemap;
	typedef map<std::string, std::string > namelinemap;
	map<std::string, vector<int> >::iterator pv;
	attlinemap fAttLine;
	namelinemap fNameLine;
	TGPopupMenu *fMenu;
	TGPopupMenu *fOptionMenu;
	TRootCanvas *fRootCanvas;   
	void *fValp[100];
	TList *fRow_lab;
	TGMrbValuesAndText *fDialog;
	
	Int_t fHitsOn;
	Int_t fHitsOff;
	TString fHitsScope;
	TString fDetHits;
	Int_t fBidHitsOn;
	Int_t fBidHitsOff;
	Int_t fBidHitsScope;
	
	Int_t fGeomOn;
	Int_t fGeomOff;
	TString fGeomScope;
	TString fDetGeom;
	Int_t fBidGeomOn;
	Int_t fBidGeomOff;
	Int_t fBidGeomScope;
	
	Int_t fThrustOnOff;
	Int_t fJetsOnOff;
	Int_t fPCutOnOff;
	Int_t fBidColorTracks;
	Float_t fPCut;
	TString fPCutScope;
	TString fThickenScope;
	Float_t fThicken;
	Int_t fBidGeom;
	Int_t fBidThrust;
	Int_t fBidJets;
	Int_t fBidPCut;
	
	TGMrbValuesAndText *fDialogCmd;
	void *fValpCmd[100];
	TList *fRow_labCmd;
	TString fEventType;
	TString fDetTypeOD;
	Int_t fBlockNrOD;
	Int_t fWithDocuOD;
	Int_t fBidEventType; 
	TString fViewType;
	Int_t fBidViewType;
	
	Int_t fBidODBlock;
	TString fODBlock;
	Int_t fBidODComplexity;
 	Int_t fODComplValue;
	
	Int_t fBidAttLine;
	TString fAttLineName;
	Int_t fBidAttLineType;
 	Int_t fAttLineType;
	Int_t fBidAttLineWidth;
 	Int_t fAttLineWidth;
	Int_t fBidAttLineColor;
 	Int_t fAttLineColor;

	Int_t fCurrentRun;
	Int_t fPreviousRun;
	Int_t fCurrentEvent;
	Int_t fPreviousEvent;
	Int_t fTotalEvents;
	Int_t fRunNr;
	Int_t fEventNr;
	stack < int > fEventNumbers;
	stack < int > fRunNumbers;
	TString fMuonFile;
	TString fElectronFile;
	TString fTauFile;
	TString fHadronFile;
	TString fCosmicsFile;
	TString fAnyEventFile;
	TGMainFrame * fFrameOGL;
	TRootCanvas * fRootC3D;
	Double_t fZoom;
	Double_t fYZCameraAngle;
	Double_t fCenter[3];
	enum EViewType { kViewXY, kViewXZ, kViewYZ, kViewSide};
	enum CmdId { kDisplayOptions, kPrintData };
	enum WinId { kIdOptions, kIdPrintOD };
	EViewType fViewDir;

public:
	void trig();
	void rotrig();
	void rewind();
	void rsevct(){ rsevct_();};
	void skip(Int_t nev=1000, Int_t nproc=0);
	void redraw() { grdraw_();};
	void next() { trig(); };
	void previous();
	void quit() {gSystem->Exit(0);};
	void getevent(Int_t ev=0, Int_t run=0);
	void SetColorTracks(Int_t docolor = 1); // Enable coloring of tracks acc to dEdx
	void printod(const char * name="CTRK", Int_t nr = 0);  // Print OD block
	void printod_withdoc(const char * name="CTRK", Int_t nr = 0);  // Print OD block
	void printodterse();  // OD summary, 1 line / track / cluster
	void hits(const char * idet = "CJ", Int_t iflag = 1, Int_t isect = 0);
	void geom(const char * idet = "CJ", Int_t iflag = 1, Int_t isect = 0);
	void recon(const char * idet = "PX" , Int_t iflag = 1, Int_t isect = 0);
	void dzdisp(const char * ctop = "REV", Int_t numid = 0, const char * mtyp = "POST");
	void dzddiv(const char * ctop = "IRODIV", const char * mtyp = "POST");
	void dedx(Int_t itrk = -1, float dedxlo=5, float dedxhi=55);
	void lego(const char * name = "ALL", float theta = 45, float phi = 45, const char * iopr = "LINZ");
	void siroll(int iladd = 0, int iflag = 0, float zmin = -9, float zmax = 9, float phmin = 0, float phmax = 6.2831997871398926);
	void file(const char * name, const char * type="DDST", 
				const char * dir="IN", Int_t recl=0,  
				const char * zopt="L   ", const char * ropt =".   ");
	void SetMuonFile(const char * fname)    {fMuonFile     = fname;};
	void SetElectronFile(const char * fname){fElectronFile = fname;};
	void SetTauFile(const char * fname)     {fTauFile      = fname;};
	void SetHadronFile(const char * fname)  {fHadronFile   = fname;};
	void SetCosmicsFile(const char * fname) {fCosmicsFile  = fname;};
	void SetAnyEventFile(const char * fname){fAnyEventFile = fname;};
			
	void muon()    {ClearEventNumbers(); file(fMuonFile.Data()    ); trig();}
	void electron(){ClearEventNumbers(); file(fElectronFile.Data()); trig();}
	void tau()     {ClearEventNumbers(); file(fTauFile.Data()     ); trig();}
	void hadron()  {ClearEventNumbers(); file(fHadronFile.Data()  ); trig();}
	void cosmic()  {ClearEventNumbers(); file(fCosmicsFile.Data() ); trig();}
	void anyevent(){ClearEventNumbers(); file(fAnyEventFile.Data()); trig();}
	
	void viewOpenGL();
	void zoom2();
	void unzoom2();
	void xyview();
	void yzview();
	void xzview();
	void sideview(Int_t plane = 0);
	void pcutonoff();
//	void SetPCut(Float_t p);
//	void SetPCutScope(const char *s) {fPCutScope = s;};
	void OpenDzdisp();
	void MomentumCut(float p = 0, const char * scope = "ALL ");
	void fillbox(Int_t igrfil=3 ,const char *  igrfed="ON  ", Int_t igrisi=3045);
	void dst(const char * btype="VRTX", Int_t clevel=2);
	void setdst();
	void helpdst();
	void setattline();
	void attline(const char * btype="CJT", Int_t type=1, Int_t width=1, Int_t color=1);
	void helpattline();
	void save_attributes(const char * filename="ms.kumac");
	Int_t getabrav(const char * name, char abrav[4]);
	void setincreaselw();
	void thrustonoff();
	void jetsonoff();
	void colorctonoff();
	//
	RopeCmds();
	~RopeCmds() {};
	void DoView();
	void StoreEventNumber();
	void HandleMouseOver(TObject* obj,UInt_t val);
	void HandleClicked(TObject* obj, UInt_t button, UInt_t state);
	void HandleCloseOGLWindow();
	void HandleClose3DWindow();
	void HandleCloseCmdWindow();
	void HandleCloseDzDispWindow();
	void RemoveMenu(TGMenuBar * menubar, const char * menu);
	void BuildMenu();
	void HandleMenu(Int_t id);
	void DisplayOptions();
	void Actions();
	void PrintODData();
	void GotoEvent();
	Int_t GetNwGNRL();
	void ClearEventNumbers();
	void ChangeBColor(TButton * b);
	void CloseDown(Int_t wid){};
	void CRButtonPressed(Int_t wid, Int_t bid, TObject *obj);
	
ClassDef(RopeCmds,0)
};

R__EXTERN RopeCmds *gRC;
R__EXTERN TView3D * gView3D;
R__EXTERN TCanvas * gCanvasDZ;
R__EXTERN TCanvas * gCanvas3D;
R__EXTERN TCanvas * gCanvasLego;
R__EXTERN TGLViewer * gGLViewer;

#endif
